<?php if ( ! defined( 'EVENT_ESPRESSO_VERSION' )) { exit(); }
/**
 * ------------------------------------------------------------------------
 *
 * Class  EE_Aim_Echeck
 *
 * @package			Event Espresso
 * @subpackage		espresso-aim-echeck
 * @author			    Brent Christensen
 * @ version		 	$VID:$
 *
 * ------------------------------------------------------------------------
 */
// define the plugin directory path and URL
define( 'EE_AIM_ECHECK_BASENAME', plugin_basename( EE_AIM_ECHECK_PLUGIN_FILE ));
define( 'EE_AIM_ECHECK_PATH', plugin_dir_path( __FILE__ ));
define( 'EE_AIM_ECHECK_URL', plugin_dir_url( __FILE__ ));
Class  EE_Aim_Echeck extends EE_Addon {

	/**
	 * class constructor
	 */
	public function __construct() {
	}

	public static function register_addon() {
		// register addon via Plugin API
		$aim_pmt_file = EE_PLUGIN_DIR_PATH . 'caffeinated' . DS . 'payment_methods' . DS . 'Aim' . DS . 'EE_PMT_Aim.pm.php';
		if( file_exists( $aim_pmt_file ) ) {
			EE_Register_Addon::register(
				'Aim_Echeck',
				array(
					'version' 					=> EE_AIM_ECHECK_VERSION,
					'min_core_version' => '4.8.41.rc.006',
					'main_file_path' 				=> EE_AIM_ECHECK_PLUGIN_FILE,
					'admin_callback' => 'additional_admin_hooks',
					// if plugin update engine is being used for auto-updates. not needed if PUE is not being used.
					'pue_options'			=> array(
						'pue_plugin_slug' => 'eea-aim-echeck',
						'plugin_basename' => EE_AIM_ECHECK_BASENAME,
						'checkPeriod' => '24',
						'use_wp_update' => FALSE,
						),
					'payment_method_paths' => array(
						EE_AIM_ECHECK_PATH . 'payment_methods' . DS . 'Aim_Echeck',
						),
					'autoloader_paths' => array(
						//we rely on the AIM gateway
						'EE_PMT_Aim' => $aim_pmt_file,
						'EEG_Aim' => EE_PLUGIN_DIR_PATH . 'caffeinated' . DS . 'payment_methods' . DS . 'Aim' . DS . 'EEG_Aim.gateway.php'
					)
			));
		} else {
			EE_Error::add_error(
				__( 'The Authorize.net AIM eCheck payment method requires the standard Authorize.net AIM payment method to be active in order to function. The standard Authorize.net payment method was not found. Are you running Event Espresso Decaf? If so, please upgrade.'), 
				__FILE__, 
				__FUNCTION__, 
				__LINE__ 
			);
		}
		
	}



	/**
	 * 	additional_admin_hooks
	 *
	 *  @access 	public
	 *  @return 	void
	 */
	public function additional_admin_hooks() {
		// is admin and not in M-Mode ?
		if ( is_admin() && ! EE_Maintenance_Mode::instance()->level() ) {
			add_filter( 'plugin_action_links', array( $this, 'plugin_actions' ), 10, 2 );
		}
	}



	/**
	 * plugin_actions
	 *
	 * Add a settings link to the Plugins page, so people can go straight from the plugin page to the settings page.
	 * @param $links
	 * @param $file
	 * @return array
	 */
	public function plugin_actions( $links, $file ) {
		if ( $file == EE_AIM_ECHECK_BASENAME ) {
			// before other links
			array_unshift( $links, '<a href="admin.php?page=espresso_payment_settings">' . __('Settings') . '</a>' );
		}
		return $links;
	}






}
// End of file EE_Aim_Echeck.class.php
// Location: wp-content/plugins/espresso-aim-echeck/EE_Aim_Echeck.class.php
