<?php

if (!defined('EVENT_ESPRESSO_VERSION')) {
	exit('No direct script access allowed');
}

/**
 *
 * EE_PMT_Onsite
 *
 *
 * @package			Event Espresso
 * @subpackage
 * @author				Mike Nelson
 *
 */
class EE_PMT_Aim_Echeck extends EE_PMT_Aim {
	
	/**
	 * Used so we still have access to the transaction during a callback filter
	 * @var EE_Transaction
	 */
	protected $_current_transaction_for_callback;

	/**
	 * Sets up a different gateway than parent
	 */
	protected function _setup_properties() {
		require_once($this->file_folder().'EEG_Aim_Echeck.gateway.php');
		$this->_gateway = new EEG_AIM_eCheck();
		$this->_pretty_name = __("Authorize.net AIM eCheck", 'event_espresso');
		$this->_default_description = __( 'Please provide the following billing information.', 'event_espresso' );
		$this->_requires_https = true;
		$this->_default_button_url = $this->file_url() . 'lib/authorize-net-echeck-logo.gif';
	}

	/**
	 * Adds the help tab
	 * @see EE_PMT_Base::help_tabs_config()
	 * @return array
	 */
	public function help_tabs_config(){
		return array(
			$this->get_help_tab_name() => array(
				'title' => __('Authorize.net AIM eCheck Settings', 'event_espresso'),
				'filename' => 'aim_echeck'
				),
		);
	}
	
	public function generate_new_billing_form( \EE_Transaction $transaction = NULL ) {
		//add a filter so we build a differenter form
		add_filter( 
			'FHEE__EE_Form_Section_Proper___construct__options_array', 
			array(
				$this,
				'change_aim_form_to_aim_echeck'
			),
			10,
			2
		);
		$this->_current_transaction_for_callback = $transaction;
		$billing_form = parent::generate_new_billing_form( $transaction );
		remove_filter( 
			'FHEE__EE_Form_Section_Proper___construct__options_array', 
			array(
				$this,
				'change_aim_form_to_aim_echeck'
			), 
			10
		);
		return $billing_form;
		
	}
	
	
	/**
	 * Intercepts the parent when it's creating the billing form to change it
	 * to the eCheck billing form
	 * @param array $options_array
	 * @param EE_Billing_Attendee_Info_Form $form_section
	 * @return array of form options
	 */
	public function change_aim_form_to_aim_echeck( $options_array, $form_section ) {
		if( isset( $options_array[ 'name' ] ) 
			&& $options_array[ 'name' ] === 'AIM_Form' ) {
			$options_array[ 'name' ] = 'AIM_eCheck_Form';
			unset( 
				$options_array[ 'subsections' ][ 'credit_card' ],
				$options_array[ 'subsections' ][ 'exp_month' ],
				$options_array[ 'subsections' ][ 'exp_year' ],
				$options_array[ 'subsections' ][ 'cvv' ] 
			);
			$default_account_name = '';
			if( $this->_current_transaction_for_callback instanceof EE_Transaction ) {
				$reg = $this->_current_transaction_for_callback->primary_registration();
				if( $reg instanceof EE_Registration
					&& $reg->attendee() instanceof EE_Attendee ) {
					$default_account_name = $reg->attendee()->full_name();
				}
			}
			$options_array[ 'subsections' ] = array_merge( 
				$options_array[ 'subsections' ],
				array(
					'account_type' => new EE_Select_Input(
						array(
							'' => __( 'Please select...', 'event_espresso' ),
							'CHECKING' => __( 'Checking', 'event_espresso' ),
							'BUSINESSCHECKING' => __( 'Business Checking', 'event_espresso' ),
							'SAVINGS' => __( 'Savings', 'event_espresso' )
						),
						array(
							'html_label_text' => __( 'Bank Account Type', 'event_espresso' ),
							'required' => true,
						)
					),
					'bank_name' => new EE_Text_Input(
						array(
							'html_label_text' => __( 'Bank Name', 'event_espresso' ),
							'required' => true,
							'validation_strategies' => array(
								new EE_Max_Length_Validation_Strategy( null, 50 )
							)
						)
					),
					'account_name' => new EE_Text_Input(
						array(
							'html_label_text' => __( 'Name on Bank Account', 'event_espresso' ),
							'required' => true,
							'validation_strategies' => array(
								new EE_Max_Length_Validation_Strategy( null, 50 )
							),
							'sensitive_data_removal_strategy' => new EE_All_Sensitive_Data_Removal(),
							'default' => $default_account_name
						)
					),
					'bank_routing_number' => new EE_Text_Input(
						array(
							'html_label_text' => __( 'Bank Routing Number (ABA code)',	'event_espresso' ),
							'required' => true,
							'validation_strategies' => array(
								new EE_Int_Validation_Strategy(),
								new EE_Max_Length_Validation_Strategy( null, 9 )
							)
						)
					),
					'account_number' => new EE_Text_Input(
						array(
							'html_label_text' => __( 'Bank Account Number', 'event_espresso' ),
							'required' => true,
							'validation_strategies' => array(
								new EE_Int_Normalization(),
								new EE_Max_Length_Validation_Strategy( null, 20 )
							),
							'sensitive_data_removal_strategy' => new EE_All_Sensitive_Data_Removal(),
						)
					),
				)
			);
			
		}
		return $options_array;
	}
	
	
	/**
	 * Completely override parent, because all the fields it autofills are for credit
	 * card purchases only
	 * @param \EE_Billing_Info_Form $billing_form
	 * @return \EE_Billing_Info_Form
	 */
	public function apply_billing_form_debug_settings( \EE_Billing_Info_Form $billing_form ) {
		if ( $this->_pm_instance->debug_mode() || $this->_pm_instance->get_extra_meta( 'test_transactions', true, false )) {
			$billing_form->get_input( 'bank_name' )->set_default( 'Wells Fargo' );
			$billing_form->get_input( 'bank_routing_number' )->set_default(  '121000248' );
			$billing_form->get_input( 'account_type' )->set_default(  'CHECKING' );
			if( ! $billing_form->get_input( 'account_name' )->raw_value() ) {
				$billing_form->get_input( 'account_name' )->set_default( 'Darth Vader' );
			}
			$billing_form->get_input( 'account_number' )->set_default( '1234567890' );
		}
		
		return $billing_form;
	}
	
	/**
	 * Extends the parent so "cvv" isn't an option for including or excluding.
	 * It doesn't make sense for eChecks
	 * @return array
	 */
	public function billing_input_names(){
		$input_names = parent::billing_input_names();
		unset( $input_names[ 'cvv' ] );
		return $input_names;
	}
}
// End of file EE_PMT_Onsite.php