<?php if ( ! defined( 'EVENT_ESPRESSO_VERSION' ) ) { exit('NO direct script access allowed'); }

/**
 *
 * EE_PMT_AuthorizeNet_Sim
 *
 *
 * @package			Event Espresso
 * @subpackage		eea-authorizenet-sim
 * @author			Event Espresso
 *
 */

class EE_PMT_AuthorizeNet_Sim extends EE_PMT_Base {


	/**
	 * path to the templates folder for the Authorize.Net PM
	 * @var string
	 */
	protected $_template_path = null;


	/**
	 * @return \EE_PMT_AuthorizeNet_Sim
	 */
	public function __construct( $pm_instance = null ) {
		require_once( $this->file_folder() . 'EEG_AuthorizeNet_Sim.gateway.php' );
		$this->_gateway = new EEG_AuthorizeNet_Sim();
		$this->_pretty_name = __("Authorize.Net SIM", 'event_espresso');
		$this->_default_description = sprintf( __( 'After clicking \'Finalize Registration\', you will be forwarded to Authorize.Net to make your payment. %1$sMake sure you return to this site in order to properly finalize your registration.%2$s', 'event_espresso' ), '<strong>', '</strong>' );
		$this->_template_path = dirname(__FILE__) . DS . 'templates' . DS;

		parent::__construct( $pm_instance );
		$this->_default_button_url = $this->file_url() . 'lib' . DS . 'authorize-net-logo.png';
	}

	
	/**
	 * Gets the form for all the settings related to this payment method type
	 *
	 * @return EE_Payment_Method_Form
	 */
	public function generate_new_settings_form() {
		$form = new EE_Payment_Method_Form( array(
			'extra_meta_inputs' => array(
				'api_login_id' => new EE_Text_Input( array(
					'html_label_text' => sprintf( __("Authorize.Net API login ID %s", "event_espresso"), $this->get_help_tab_link() ),
					'required' => true
				)),
				'transaction_key' => new EE_Text_Input( array(
					'html_label_text' => sprintf( __("Authorize.Net Transaction Key %s", "event_espresso"), $this->get_help_tab_link() ),
					'required' => true
				)),
				'test_transactions_yn'=>new EE_Yes_No_Input(
					array(
						'html_label_text'=> sprintf( __("Send Test transactions? %s", 'event_espresso'),  $this->get_help_tab_link() ),
						'html_help_text'=> __("Send Test transactions, even to live server.", 'event_espresso'),
						'default' => false,
						'required' => true
					)
				)
			)
		));

		return $form;
	}


	/**
	 * Just to post some debug info on the checkour page.
	 *
	 * @param \EE_Transaction $transaction
	 * @return null
	 */
	public function generate_new_billing_form( EE_Transaction $transaction = null ) {
		$settings = $this->_pm_instance->settings_array();
		$form = new EE_Billing_Info_Form(
			$this->_pm_instance,
			array(
				'name' => 'AuthorizeNet_Sim_Info_Form',
				'html_id'=> 'authorizenet-sim-info-form',
				'subsections' => array(
					new EE_Form_Section_Proper(
						array(
							'layout_strategy' => new EE_Template_Layout(
								array(
									'layout_template_file' => $this->_template_path . 'authorizenet_sim_debug_info.template.php',
									'template_args' => array( 'debug_mode' => $this->_pm_instance->debug_mode(), 'test_transactions' => $settings['test_transactions_yn'] )
								)
							)
						)
					)
				)
			)
		);

		return $form;
	}


	public function help_tabs_config() {
		return array(
			$this->get_help_tab_name() => array(
				'title' => __('Authorize.Net SIM Settings', 'event_espresso'),
				'filename' => 'payment_methods_overview_authorizenet_sim'
			),
		);
	}
}

// End of file EE_PMT_AuthorizeNet_Sim.pm.php
