<?php if ( ! defined( 'EVENT_ESPRESSO_VERSION' )) { exit('NO direct script access allowed'); }

// Define the plugin directory path and URL.
define( 'EEA_BRAINTREE_PLUGIN_BASENAME', plugin_basename( EEA_BRAINTREE_PLUGIN_FILE ) );
define( 'EEA_BRAINTREE_PLUGIN_URL', plugin_dir_url( EEA_BRAINTREE_PLUGIN_FILE ) );
define( 'EEA_BRAINTREE_API_PATH', EEA_BRAINTREE_PLUGIN_PATH . 'includes' . DS . 'braintree-3.21.0' . DS );

/**
 * Class  EE_Braintree_Gateway
 *
 * @package			Event Espresso
 * @subpackage		espresso-braintree-gateway
 * @author			Event Espresso
 * @version 		$VID:$
 */
class EE_Braintree_Gateway extends EE_Addon {


    /**
     *    Register addon.
     *
     * @access public
     * @return void
     * @throws \EE_Error
     */
	public static function register_addon() {
		// Register addon via EE Plugin API.
		EE_Register_Addon::register(
			'Braintree_Gateway',
			array(
				'version' => EEA_BRAINTREE_VERSION,
				'min_core_version' => EEA_BRAINTREE_MIN_CORE_VERSION,
				'main_file_path' => EEA_BRAINTREE_PLUGIN_FILE,
				'admin_callback' => 'additional_braintree_admin_hooks',
				// Register autoloaders
				'autoloader_paths' => array(
					'EE_PMT_Base' => EE_LIBRARIES . 'payment_methods' . DS . 'EE_PMT_Base.lib.php',
					'EE_PMT_Braintree_Dropin' => EEA_BRAINTREE_PLUGIN_PATH . 'payment_methods' . DS . 'Braintree_Dropin' . DS . 'EE_PMT_Braintree_Dropin.pm.php',
				),
				// If plugin update engine is being used for auto-updates. Not needed if PUE is not being used.
				'pue_options' => array(
					'pue_plugin_slug' => 'eea-braintree-gateway',
					'plugin_basename' => EEA_BRAINTREE_PLUGIN_BASENAME,
					'checkPeriod' => '24',
					'use_wp_update' => FALSE,
				),
				'payment_method_paths' => array(
					EEA_BRAINTREE_PLUGIN_PATH . 'payment_methods' . DS . 'Braintree_Dropin'
				),
			)
		);
	}



    /**
     * a safe space for addons to add additional logic like setting hooks
     * that will run immediately after addon registration
     * making this a great place for code that needs to be "omnipresent"
     */
    public function after_registration()
    {
        // Log Braintree JS errors.
        add_action('wp_ajax_eea_braintree_log_error', array('EE_PMT_Braintree_Dropin', 'log_braintree_error'));
        add_action('wp_ajax_nopriv_eea_braintree_log_error', array('EE_PMT_Braintree_Dropin', 'log_braintree_error'));
        $psr4_loader = \EE_Psr4AutoloaderInit::psr4_loader();
        $psr4_loader->addNamespace(
            'EEA_Braintree',
            EEA_BRAINTREE_API_PATH . 'lib' . DS
        );
    }



    /**
     *    Setup default data for the addon.
     *
     * @access public
     * @return void
     * @throws \EE_Error
     */
	public function initialize_default_data() {
		parent::initialize_default_data();

		// Update the currencies supported by this gateway (if changed).
		$braintree = EEM_Payment_method::instance()->get_one_of_type( 'Braintree_Dropin' );
		// Update If the payment method already exists.
		if ( $braintree ) {
			$currencies = $braintree->get_all_usable_currencies();
			$all_related = $braintree->get_many_related( 'Currency' );

			if ($currencies !== $all_related) {
				$braintree->_remove_relations( 'Currency' );
				foreach ( $currencies as $currency_obj ) {
					$braintree->_add_relation_to( $currency_obj, 'Currency' );
				}
			}
		}
	}


	/**
	 * 	Additional admin hooks.
	 *
	 *  @access public
	 *  @return void
	 */
	public static function additional_braintree_admin_hooks() {
		// Is admin and not in M-Mode ?
		if ( is_admin() && ! EE_Maintenance_Mode::instance()->level() ) {
			add_filter( 'plugin_action_links', array( 'EE_Braintree_Gateway', 'plugin_actions' ), 10, 2 );
		}
	}


	/**
	 * Add a settings link to the Plugins page.
	 *
	 * Add a settings link to the Plugins page, so people can go straight from the plugin page to the settings page.
	 * @param $links
	 * @param $file
	 * @return array
	 */
	public static function plugin_actions( $links, $file ) {
		if ( $file === EEA_BRAINTREE_PLUGIN_BASENAME ) {
			// Before other links
			array_unshift( $links, '<a href="admin.php?page=espresso_payment_settings">' . __('Settings') . '</a>' );
		}
		return $links;
	}
}
// End of file EE_Braintree_Gateway.class.php
// Location: wp-content/plugins/espresso-braintree-gateway/EE_Braintree_Gateway.class.php