<?php
namespace Test\Integration;

require_once dirname(__DIR__) . '/Setup.php';

use Test\Setup;
use EEA_Braintree;

class HttpTest extends Setup
{
    public function testProductionSSL()
    {
        try {
            EEA_Braintree\Configuration::environment('production');
            $this->setExpectedException('EEA_Braintree\Exception\Authentication');
            $http = new EEA_Braintree\Http(EEA_Braintree\Configuration::$global);
            $http->get('/');
        } catch (EEA_Braintree\Exception $e) {
            EEA_Braintree\Configuration::environment('development');
            throw $e;
        }
        EEA_Braintree\Configuration::environment('development');
    }

    public function testSandboxSSL()
    {
        try {
            EEA_Braintree\Configuration::environment('sandbox');
            $this->setExpectedException('EEA_Braintree\Exception\Authentication');
            $http = new EEA_Braintree\Http(EEA_Braintree\Configuration::$global);
            $http->get('/');
        } catch (EEA_Braintree\Exception $e) {
            EEA_Braintree\Configuration::environment('development');
            throw $e;
        }
        EEA_Braintree\Configuration::environment('development');
    }

    public function testSandboxSSLWithExplicitVersionSet()
    {
        try {
            EEA_Braintree\Configuration::environment('sandbox');
            EEA_Braintree\Configuration::sslVersion(6);
            $this->setExpectedException('EEA_Braintree\Exception\Authentication');
            $http = new EEA_Braintree\Http(EEA_Braintree\Configuration::$global);
            $http->get('/');
        } catch (EEA_Braintree\Exception $e) {
            EEA_Braintree\Configuration::environment('development');
            EEA_Braintree\Configuration::sslVersion(null);
            throw $e;
        }
        EEA_Braintree\Configuration::environment('development');
        EEA_Braintree\Configuration::sslVersion(null);
    }

    public function testSandboxSSLFailsWithIncompatibleSSLVersion()
    {
        try {
            EEA_Braintree\Configuration::environment('sandbox');
            EEA_Braintree\Configuration::sslVersion(3);
            $this->setExpectedException('EEA_Braintree\Exception\SSLCertificate', null, 35);
            $http = new EEA_Braintree\Http(EEA_Braintree\Configuration::$global);
            $http->get('/');
        } catch (EEA_Braintree\Exception $e) {
            EEA_Braintree\Configuration::environment('development');
            EEA_Braintree\Configuration::sslVersion(null);
            throw $e;
        }
        EEA_Braintree\Configuration::environment('development');
        EEA_Braintree\Configuration::sslVersion(null);
    }

    public function testSslError()
    {
        try {
            EEA_Braintree\Configuration::environment('sandbox');
            $this->setExpectedException('EEA_Braintree\Exception\SSLCertificate', null, 3);
            $http = new EEA_Braintree\Http(EEA_Braintree\Configuration::$global);
            $http->_doUrlRequest('get', '/malformed_url');
        } catch (EEA_Braintree\Exception $e) {
            EEA_Braintree\Configuration::environment('development');
            throw $e;
        }
        EEA_Braintree\Configuration::environment('development');
    }

    public function testAcceptGzipEncodingSetFalse()
    {
        $originalGzipEncoding = EEA_Braintree\Configuration::acceptGzipEncoding();
        EEA_Braintree\Configuration::acceptGzipEncoding(false);
        try {
            $result = EEA_Braintree\Customer::create([
                'firstName' => 'Mike',
                'lastName' => 'Jones',
                'company' => 'Jones Co.',
                'email' => 'mike.jones@example.com',
                'phone' => '419.555.1234',
                'fax' => '419.555.1235',
                'website' => 'http://example.com'
                ]);
            $this->assertEquals(true, $result->success);
            $customer = $result->customer;
            $this->assertEquals('Mike', $customer->firstName);
        } catch(EEA_Braintree\Exception $e) {
            EEA_Braintree\Configuration::acceptGzipEncoding($originalGzipEncoding);
            throw $e;
        }
        EEA_Braintree\Configuration::acceptGzipEncoding($originalGzipEncoding);
    }

    public function testAcceptGzipEncodingSetToTrue()
    {
        $originalGzipEncoding = EEA_Braintree\Configuration::acceptGzipEncoding();
        EEA_Braintree\Configuration::acceptGzipEncoding(true);
        try {
            $result = EEA_Braintree\Customer::create([
                'firstName' => 'Mike',
                'lastName' => 'Jones',
                'company' => 'Jones Co.',
                'email' => 'mike.jones@example.com',
                'phone' => '419.555.1234',
                'fax' => '419.555.1235',
                'website' => 'http://example.com'
                ]);
            $this->assertEquals(true, $result->success);
            $customer = $result->customer;
            $this->assertEquals('Mike', $customer->firstName);
        } catch(EEA_Braintree\Exception $e) {
            EEA_Braintree\Configuration::acceptGzipEncoding($originalGzipEncoding);
            throw $e;
        }
        EEA_Braintree\Configuration::acceptGzipEncoding($originalGzipEncoding);
    }

    public function testAuthorizationWithConfig()
    {
        $config = new EEA_Braintree\Configuration([
            'environment' => 'development',
            'merchant_id' => 'integration_merchant_id',
            'publicKey' => 'badPublicKey',
            'privateKey' => 'badPrivateKey'
        ]);

        $http = new EEA_Braintree\Http($config);
        $result = $http->_doUrlRequest('GET', $config->baseUrl() . '/merchants/integration_merchant_id/customers');
        $this->assertEquals(401, $result['status']);
    }
}
