<?php
namespace Test\Unit;

require_once dirname(__DIR__) . '/Setup.php';

use Test\Setup;
use EEA_Braintree;

class HttpTest extends Setup
{
    public function testMalformedNoSsl()
    {
        try {
            EEA_Braintree\Configuration::environment('development');
            $this->setExpectedException('EEA_Braintree\Exception\Connection', null, 3);
            $http = new EEA_Braintree\Http(EEA_Braintree\Configuration::$global);
            $http->_doUrlRequest('get', '/a_malformed_url');
        } catch (EEA_Braintree\Exception $e) {
            throw $e;
        }
    }

    public function testMalformedUrlUsingSsl()
    {
        try {
            EEA_Braintree\Configuration::environment('sandbox');
            $this->setExpectedException('EEA_Braintree\Exception\SSLCertificate', null, 3);
            $http = new EEA_Braintree\Http(EEA_Braintree\Configuration::$global);
            $http->_doUrlRequest('get', '/a_malformed_url_using_ssl');
        } catch (EEA_Braintree\Exception $e) {
            EEA_Braintree\Configuration::environment('development');
            throw $e;
        }
        EEA_Braintree\Configuration::environment('development');
    }

    public function testSSLVersionError()
    {
        try {
            EEA_Braintree\Configuration::environment('sandbox');
            EEA_Braintree\Configuration::sslVersion(3);
            $this->setExpectedException('EEA_Braintree\Exception\SSLCertificate', null, 35);
            $http = new EEA_Braintree\Http(EEA_Braintree\Configuration::$global);
            $http->get('/');
        } catch (EEA_Braintree\Exception $e) {
            EEA_Braintree\Configuration::environment('development');
            EEA_Braintree\Configuration::sslVersion(null);
            throw $e;
        }
        EEA_Braintree\Configuration::environment('development');
        EEA_Braintree\Configuration::sslVersion(null);
    }

    public function testGoodRequest()
    {
        EEA_Braintree\Configuration::environment('development');
        $http = new EEA_Braintree\Http(EEA_Braintree\Configuration::$global);
        $http->_doUrlRequest('get', 'http://example.com');
    }
}