<?php if ( ! defined( 'EVENT_ESPRESSO_VERSION' )) { exit('NO direct script access allowed'); }


/**
 * Class  EEG_Braintree_Dropin
 *
 * @package			Event Espresso
 * @subpackage		espresso-braintree-gateway
 * @author			Event Espresso
 * @version 		$VID:$
 */
class EEG_Braintree_Dropin extends EE_Onsite_Gateway {

	/**
	 * Braintree user id.
	 * @var $_merchant_id string
	 */
	protected $_merchant_id = NULL;

	/**
	 * Braintree public key.
	 * @var $_public_key string
	 */
	protected $_public_key = NULL;

	/**
	 * Braintree private key.
	 * @var $_private_key string
	 */
	protected $_private_key = NULL;

	/**
	 *	Currencies supported by this gateway.
	 *	@var array
	 */
	protected $_currencies_supported = EE_Gateway::all_currencies_supported;


	/**
	 * @return EE_Onsite_Gateway
	 */
	public function __construct() {
		// Include Braintree Drop-in UI dependencies.
		require_once( EEA_BRAINTREE_API_PATH . 'lib' . DS . 'Braintree.php' );

		parent::__construct();
	}


	/**
	 * Process the payment.
	 *
	 * @param EEI_Payment $payment
	 * @param array  $billing_info
	 * @return EE_Payment|EEI_Payment
	 */
	public function do_direct_payment( $payment, $billing_info = null ) {
		if (! $payment instanceof EEI_Payment) {
			$payment->set_gateway_response(__('Error. No associated payment was found.', 'event_espresso'));
			$payment->set_status($this->_pay_model->failed_status());
			return $payment;
		}
		$transaction = $payment->transaction();
		if (! $transaction instanceof EE_Transaction) {
			$payment->set_gateway_response(__('Could not process this payment because it has no associated transaction.', 'event_espresso'));
			$payment->set_status($this->_pay_model->failed_status());
			return $payment;
		}

		// Check for the payment nonce.
		if ( empty($billing_info['braintree_pm_nonce']) ) {
			$payment->set_gateway_response( 'Payment nonce is needed to process the transaction. No or incorrect payment nonce provided.' );
			$payment->set_status( $this->_pay_model->failed_status() );
			$this->log( array('Braintree Exception:' => 'No Payment Nonce provided !'), $payment );
			return $payment;
		}

		//pull the details for the primary registrant.
		$primary_registration = $transaction->primary_registration();
		$primary_attendee = $primary_registration->attendee();

		// Setup Sale Request data.
		$sale_params = array(
			'amount' => str_replace( array(','), '', number_format( $payment->amount(), 2) ),
			'paymentMethodNonce' => $billing_info['braintree_pm_nonce'],
			'options' => array(
				'submitForSettlement' => true
			),
			'customer' => array(
				'firstName' => isset( $billing_info['first_name'] ) ? $billing_info['first_name'] : $primary_attendee->fname(),
				'lastName' => isset( $billing_info['last_name'] ) ? $billing_info['last_name'] : $primary_attendee->lname(),
				'phone' => isset( $billing_info['phone'] ) ? $billing_info['phone'] :  $primary_attendee->phone(),
				'email' => isset( $billing_info['email'] ) ? $billing_info['email'] : $primary_attendee->email(),
			),
			'billing' => array(
				'firstName' => isset( $billing_info['first_name'] ) ? $billing_info['first_name'] : $primary_attendee->fname(),
				'lastName' => isset( $billing_info['last_name'] ) ? $billing_info['last_name'] : $primary_attendee->lname(),
				'streetAddress' => isset( $billing_info['address'] ) ? $billing_info['address'] : '',
				'extendedAddress' => isset( $billing_info['address2'] ) ? $billing_info['address2'] : '',
				'locality' => isset( $billing_info['city'] ) ? $billing_info['city'] : '',
				'region' => isset( $billing_info['state'] ) ? $billing_info['state'] : '',
				'postalCode' => isset( $billing_info['zip'] ) ? $billing_info['zip'] : '',
				'countryCodeAlpha2' => isset( $billing_info['country'] ) ? $billing_info['country'] : ''
			),
			'channel' => 'EventEspresso_SP'
		);

		// Filter the Sale parameters.
		$sale_params = apply_filters('FHEE__EEG_Braintree_Dropin__do_direct_payment__sale_parameters', $sale_params, $transaction);

		try {
			// Configure Braintree environment.
			$environment = ( $this->_debug_mode ) ? 'sandbox' : 'production';
			EEA_Braintree\Configuration::environment( $environment );
			EEA_Braintree\Configuration::merchantId( $this->_merchant_id );
			EEA_Braintree\Configuration::publicKey( $this->_public_key );
			EEA_Braintree\Configuration::privateKey( $this->_private_key );
			// Log and Do a Sale request.
			$this->_log_clean_request( $sale_params, $payment, 'Braintree Sale' );
			$result = EEA_Braintree\Transaction::sale( $sale_params );
			$this->_log_clean_response( $result, $payment, 'Braintree Sale request Response' );
		} catch ( \Exception $e ) {
			$ex_msg = $e->getMessage();
			if ( empty($ex_msg) ) {
				$ex_msg = get_class($e);
			}
			$payment->set_gateway_response( $ex_msg );
			$payment->set_status( $this->_pay_model->failed_status() );
			$this->log( array('Braintree Exception:' => $ex_msg), $payment );
			return $payment;
		}

		// Update payment depending on the request result.
		if ( $result instanceof EEA_Braintree\Result\Successful ) {
			if ( isset($result->transaction->amount) && ! empty($result->transaction->amount) ) {
				$payment->set_amount( floatval($result->transaction->amount) );
			}
			$payment->set_status( $this->_pay_model->approved_status() );
			$payment->set_txn_id_chq_nmbr( isset( $result->transaction->id ) ? $result->transaction->id : null );
			$payment->set_gateway_response( $result->transaction->status );
			$payment->set_details( $result->transaction->creditCard );
		} elseif ( $result instanceof EEA_Braintree\Result\Error ) {
			$payment->set_status( $this->_pay_model->declined_status() );
			$response_msg = $result->message;
			$response_msg .= ( ! empty($result->transaction->processorSettlementResponseCode) ) ? ' : ' . $result->transaction->processorSettlementResponseCode : '';
			$response_msg .= ( ! empty($result->transaction->processorSettlementResponseText) ) ? ' : ' . $result->transaction->processorSettlementResponseText : '';
			$payment->set_gateway_response( $response_msg );
			$payment->set_txn_id_chq_nmbr( isset( $result->transaction->id ) ? $result->transaction->id : null );
		} else {
			$payment->set_status( $this->_pay_model->failed_status() );
			$payment->set_gateway_response( __( 'Undefined gateway response.', 'event_espresso' ) );
		}

		return $payment;
	}


	/**
	 * CLeans out sensitive data and then logs it.
	 *
	 * @param array $request
	 * @param EEI_Payment $payment
	 * @param string $data_info
	 * @return void
	 */
	private function _log_clean_request( $request, $payment, $data_info ) {
		$cleaned_request_data = $request;
		unset( $cleaned_request_data['paymentMethodNonce'] );
		$this->log( array($data_info => $cleaned_request_data), $payment );
	}


	/**
	 * Log a cleared response.
	 *
	 * @param array $response
	 * @param EEI_Payment $payment
	 * @param string $data_info
	 * @return array cleaned
	 */
	private function _log_clean_response( $response, $payment, $data_info ) {
		$this->log( array($data_info => $response), $payment );
		return $response;
	}
}