<?php if ( ! defined( 'EVENT_ESPRESSO_VERSION' )) { exit('NO direct script access allowed'); }


/**
 * Class  EE_PMT_Braintree_Dropin
 *
 * @package			Event Espresso
 * @subpackage		espresso-braintree-gateway
 * @author			Event Espresso
 * @version 		$VID:$
 */
class EE_PMT_Braintree_Dropin extends EE_PMT_Base {


	/**
	 *  Path to the templates folder for the Braintree payment method.
	 * @var string
	 */
	protected $_template_path = NULL;


	/**
	 *  Class constructor.
	 *
	 * @param EE_Payment_Method $pm_instance
	 * @throws \EE_Error
	 * @return \EE_PMT_Braintree_Dropin
	 */
	public function __construct( $pm_instance = NULL ) {
		$this->_template_path = dirname( __FILE__ ) . DS . 'templates' . DS;
		$this->_default_description = __( 'Please provide the following billing information.', 'event_espresso' );
		$this->_pretty_name = __( 'Braintree', 'event_espresso' );
		$this->_cache_billing_form = TRUE;
		$this->_requires_https = TRUE;

		// Load gateway.
		require_once( EEA_BRAINTREE_PLUGIN_PATH . 'payment_methods' . DS . 'Braintree_Dropin' . DS . 'EEG_Braintree_Dropin.gateway.php' );
		$this->_gateway = new EEG_Braintree_Dropin();

		parent::__construct( $pm_instance );

		// Scripts for Braintree Drop-in.
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_braintree_payment_scripts' ) );
	}


	/**
	 *  Generate a new payment settings form.
	 *
	 * @return EE_Payment_Method_Form
	 */
	public function generate_new_settings_form() {
		$pms_form_args = array(
			'extra_meta_inputs' => array(
				'merchant_id' => new EE_Text_Input( array(
					'html_label_text' => sprintf( __('Merchant ID: %1$s', 'event_espresso'), $this->get_help_tab_link() ),
					'required' => true
				)),
				'public_key' => new EE_Text_Input( array(
					'html_label_text' => sprintf( __('Public key: %1$s', 'event_espresso'), $this->get_help_tab_link() ),
					'required' => true
				)),
				'private_key' => new EE_Text_Input( array(
					'html_label_text' => sprintf( __('Private key: %1$s', 'event_espresso'), $this->get_help_tab_link() ),
					'required' => true
				))
			)
		);

		// Filtering the form contents.
		$pms_form_args = apply_filters('FHEE__EE_PMT_Braintree_Dropin__generate_new_settings_form__form_filtering', $pms_form_args, $this, $this->_pm_instance);
		$pms_form = new EE_Payment_Method_Form($pms_form_args);
		return $pms_form;
	}


	/**
	 *  Creates a billing form for this payment method type.
	 *
	 * @param \EE_Transaction $transaction
	 * @return \EE_Billing_Info_Form
	 */
	public function generate_new_billing_form( EE_Transaction $transaction = NULL, $extra_args = array() ) {
		$form = new EE_Billing_Attendee_Info_Form(
			$this->_pm_instance,
			array(
				'name' => 'Braintree_Dropin_Billing_Form',
				'html_id' => 'eea-braintree-billing-form',
				'html_class' => 'ee-billing-form',
				'subsections' => array(
					'braintree_pm_nonce' => new EE_Hidden_Input( array(
						'html_id' => 'eea-braintree-pm-nonce',
						'default' => null,
						'sensitive_data_removal_strategy' => new EE_All_Sensitive_Data_Removal()
					)),
					'braintree_dropin_form' => $this->braintree_dropin_form()
				)
			)
		);

		return $this->generate_billing_form_debug_content( $form );
	}


	/**
	 *  Use Drop-in form.
	 *
	 * @return EE_Form_Section_Proper
	 */
	public function braintree_dropin_form() {
		$template_args = array();
		return new EE_Form_Section_Proper(
			array(
				'layout_strategy' => new EE_Template_Layout(
					array(
						'layout_template_file' => $this->_template_path . 'braintree_dropin_form.template.php',
						'template_args' => $template_args
					)
				)
			)
		);
	}


	/**
	 *  Possibly adds debug content to the billing form.
	 *
	 * @param \EE_Billing_Info_Form $billing_form
	 * @return \EE_Billing_Info_Form
	 */
	public function generate_billing_form_debug_content( EE_Billing_Info_Form $billing_form ) {
		if ( $this->_pm_instance->debug_mode()  || $this->_pm_instance->get_extra_meta( 'test_transactions', TRUE, FALSE ) ) {
			$billing_form->add_subsections(
				array( 'fyi_about_autofill' => $billing_form->payment_fields_autofilled_notice_html() ),
				'account_type'
			);
			$billing_form->add_subsections(
				array( 'debug_content' => new EE_Form_Section_HTML_From_Template( $this->_template_path . 'braintree_debug_info.template.php' )),
				'account_type'
			);
		}
		return $billing_form;
	}


	/**
	 *  Load all the scripts needed for the Braintree Drop-in checkout.
	 *
	 * @return void
	 */
	public function enqueue_braintree_payment_scripts() {
		// Configure Braintree environment.
		$environment = ( $this->_pm_instance->debug_mode() ) ? 'sandbox' : 'production';
		$config_e = '';
		$client_token = '';
		try {
			EEA_Braintree\Configuration::environment( $environment );
			EEA_Braintree\Configuration::merchantId( $this->_pm_instance->get_extra_meta( 'merchant_id', TRUE ) );
			EEA_Braintree\Configuration::publicKey( $this->_pm_instance->get_extra_meta( 'public_key', TRUE ) );
			EEA_Braintree\Configuration::privateKey( $this->_pm_instance->get_extra_meta( 'private_key', TRUE ) );
			$client_token = EEA_Braintree\ClientToken::generate();
		} catch ( \Exception $e ) {
			$config_e = get_class($e);
		}

		wp_enqueue_script( 'eea_braintree_js', EEA_BRAINTREE_PLUGIN_URL . 'scripts' . DS . 'braintree.js', array( 'single_page_checkout' ), '2.15.2', true );
		wp_enqueue_script( 'espresso_braintree_dropin_js', EEA_BRAINTREE_PLUGIN_URL . 'scripts' . DS . 'eea_braintree_dropin_gtw.js', array( 'eea_braintree_js', 'single_page_checkout' ), EEA_BRAINTREE_VERSION, true );
		$trans_args = array(
			'clientToken' => $client_token,
			'card_error_message' => __( 'Payment Error! Please refresh the page and try again or contact support.', 'event_espresso' ),
			'no_braintree_js_error' => __( 'It appears the Braintree Checkout javascript was not loaded properly! Please refresh the page and try again or contact support.', 'event_espresso' ),
			'braintree_config_exception' => $config_e,
			'braintree_config_e_message' =>  __( 'There was a problem in trying to Configure Braintree. Please ensure that you are using a right account for this environment.', 'event_espresso' ),
			'payment_method_slug' => $this->_pm_instance->slug(),
		);

		// Localize the script with our transaction data.
		wp_localize_script( 'espresso_braintree_dropin_js', 'braintree_transaction_args', $trans_args);
	}


	/**
	 *  Adds a help tab.
	 *
	 * @see EE_PMT_Base::help_tabs_config()
	 * @return array
	 */
	public function help_tabs_config() {
		return array(
			$this->get_help_tab_name() => array(
				'title' => __( 'Braintree Settings', 'event_espresso' ),
				'filename' => 'payment_methods_overview_braintree'
			)
		);
	}


	/**
	 *  Log TXN Error.
	 *
	 * @return void
	 */
	public static function log_braintree_error() {
		if ( isset($_POST['txn_id']) && ! empty($_POST['txn_id']) ) {
			$braintree_pm = EEM_Payment_method::instance()->get_one_of_type( 'Braintree_Dropin' );
			$transaction = EEM_Transaction::instance()->get_one_by_ID( $_POST['txn_id'] );
			$braintree_pm->type_obj()->get_gateway()->log( array('JS Error (Transaction: ' . $transaction->ID() . ')' => $_POST['message']), $transaction );
		}
	}


	/**
	 * Adjust the billing form data.
	 * Need Country and State abbreviations, not full names.
	 *
	 * @param EE_Billing_Info_Form $billing_form
	 * @return array
	 */
	protected function _get_billing_values_from_form( $billing_form ) {
		$billing_values = parent::_get_billing_values_from_form( $billing_form );
		if( ! empty( $billing_values[ 'country' ] ) && $billing_form->get_input( 'country' ) instanceof EE_Country_Select_Input ) {
			$billing_values[ 'country' ] = $billing_form->get_input_value( 'country' );
		}
		if( ! empty( $billing_values[ 'state' ] ) && $billing_form->get_input( 'state' ) instanceof EE_State_Select_Input )  {
			$state = EEM_State::instance()->get_col( array( array('STA_name' => $billing_values['state']), 'limit' => 1), 'STA_abbrev' );
			$billing_values[ 'state' ] = $state[0];
		}
		return $billing_values;
	}
}