jQuery(document).ready( function( $ ) {

	var EEA_BRAINTREE_PM;

	/**
	 * @namespace EEA_BRAINTREE_PM
	 * @type {{
	 *		handler : object,
	 *		txn_data : object,
	 *		selected : boolean,
	 *		initialized : boolean,
	 *		notification : string,
	 *		pm_nonce : object,
	 *		checkout : object,
	 *		spco_payment_form : object,
	 *		braintree_button_div : object,
	 *		submit_payment_button : object,
	 *		braintree_submit_flag : object,
	 *		client_token_container : object,
	 *		payment_method_selector : object,
	 *		payment_method_info_div : object,
	 *		offset_from_top_modifier : integer
	 * }}
	 *
	 * @namespace braintree_transaction_args
	 * @type {{
	 * 		clientToken : string,
	 * 		card_error_message : string,
	 * 		no_braintree_js_error : string,
	 * 		braintree_config_exception : string,
	 * 		braintree_config_e_message : string
	 * }}
	 */
	EEA_BRAINTREE_PM = {

		handler : {},
		txn_data : {},
		selected : false,
		initialized : false,
		notification : '',
		pm_nonce : {},
		checkout : {},
		spco_payment_form : {},
		braintree_button_div : {},
		submit_payment_button : {},
		client_token_container : {},
		payment_method_selector : {},
		payment_method_info_div : {},
		offset_from_top_modifier : -400,


		/**
		 * @function initialize
		 */
		initialize : function() {
			EEA_BRAINTREE_PM.initialize_objects();

			// Ensure that the SPCO js class is loaded.
			if ( typeof SPCO === 'undefined' ) {
				EEA_BRAINTREE_PM.hide_braintree();
				EEA_BRAINTREE_PM.display_error( braintree_transaction_args.no_SPCO_error, 'SPCO undefined' );
				return;
			}

			// Is Braintree Config initialized.
			if ( braintree_transaction_args.braintree_config_exception.trim() ) {
				var e_a_msg = braintree_transaction_args.braintree_config_exception;
				if ( braintree_transaction_args.braintree_config_exception.indexOf('Authentication') > -1 ) {
					e_a_msg = e_a_msg + ' : ' + braintree_transaction_args.braintree_config_e_message;
				}
				EEA_BRAINTREE_PM.submit_payment_button.prop( 'disabled', true ).addClass( 'spco-disabled-submit-btn' );
				SPCO.offset_from_top_modifier = EEA_BRAINTREE_PM.offset_from_top_modifier;
				EEA_BRAINTREE_PM.notification = SPCO.generate_message_object( '', SPCO.tag_message_for_debugging( 'EEA_Braintree.Configuration() error', e_a_msg ), '' );
				SPCO.scroll_to_top_and_display_messages( EEA_BRAINTREE_PM.braintree_button_div, EEA_BRAINTREE_PM.notification, true );
				return;
			}

			// Ensure that the Braintree Dropin JS was loaded.
			if ( typeof braintree === 'undefined' ) {
				SPCO.offset_from_top_modifier = EEA_BRAINTREE_PM.offset_from_top_modifier;
				EEA_BRAINTREE_PM.notification = SPCO.generate_message_object( '', SPCO.tag_message_for_debugging( 'EEA_BRAINTREE_PM.init() error', braintree_transaction_args.no_braintree_js_error ), '' );
				SPCO.scroll_to_top_and_display_messages( EEA_BRAINTREE_PM.braintree_button_div, EEA_BRAINTREE_PM.notification, true );
				return;
			}
			// Braintree selected ?
			if ( ! EEA_BRAINTREE_PM.submit_payment_button.length ) {
				return;
			}

			EEA_BRAINTREE_PM.selected = true;
			EEA_BRAINTREE_PM.disable_SPCO_submit_buttons_if_Braintree_selected();

			// Already initialized?
			if ( EEA_BRAINTREE_PM.initialized ) {
				// Just load the Braintree Dropin form.
				EEA_BRAINTREE_PM.setup_braintree();
				return;
			}

			EEA_BRAINTREE_PM.get_transaction_data();
			EEA_BRAINTREE_PM.setup_listeners();
			EEA_BRAINTREE_PM.initialized = true;
		},


		/**
		 * @function initialize_objects
		 */
		initialize_objects : function() {
			EEA_BRAINTREE_PM.payment_method_selector = $( '#ee-available-payment-method-inputs-braintree_dropin-lbl' );
			EEA_BRAINTREE_PM.payment_method_info_div = $( '#spco-payment-method-info-braintree_dropin' );
			EEA_BRAINTREE_PM.client_token_container = $( '#spcoeea-braintree-dropin-client_token' );
			EEA_BRAINTREE_PM.dropin_container = $( '#eea-braintree-dropin-container' );
			EEA_BRAINTREE_PM.spco_payment_form = $( '#ee-spco-payment_options-reg-step-form' );
			EEA_BRAINTREE_PM.submit_payment_button = $( '#eea-braintree-di-button-btn' );
			EEA_BRAINTREE_PM.braintree_button_div = $( '#eea-braintree-di-button-dv' );
			EEA_BRAINTREE_PM.pm_nonce = $( '#eea-braintree-pm-nonce' );
		},


		/**
		 * @function setup_listeners
		 */
		setup_listeners : function() {
			EEA_BRAINTREE_PM.set_listener_for_payment_method_selector();
			EEA_BRAINTREE_PM.set_listener_for_payment_notice();
		},


		/**
		 * @function get_transaction_data
		 */
		get_transaction_data : function() {
			var req_data = {};
			req_data.step = 'payment_options';
			req_data.action = 'get_transaction_details_for_gateways';
			req_data.selected_method_of_payment = braintree_transaction_args.payment_method_slug;
			req_data.generate_reg_form = 0;
			req_data.process_form_submission = 0;
			req_data.noheader = 1;
			req_data.ee_front_ajax = 1;
			req_data.EESID = eei18n.EESID;
			req_data.revisit = eei18n.revisit;
			req_data.e_reg_url_link = eei18n.e_reg_url_link;

			$.ajax( {
				type : "POST",
				url : eei18n.ajax_url,
				data : req_data,
				dataType : "json",

				beforeSend : function() {
					SPCO.do_before_sending_ajax();
				},
				success : function( response ) {
				    SPCO.end_ajax();
					// If we can't get a tranіaction data we can't set up a checkout.
					if ( response['error'] || typeof response['TXN_ID'] == 'undefined' || response['TXN_ID'] == null ) {
						return SPCO.submit_reg_form_server_error();
					}
					EEA_BRAINTREE_PM.txn_data = response;
					// Load Braintree Dropin form.
					EEA_BRAINTREE_PM.setup_braintree();
				},
				error : function() {
					SPCO.end_ajax();
					return SPCO.submit_reg_form_server_error();
				}
			});
		},


		/**
		 * @function setup_braintree
		 */
		setup_braintree : function() {
			EEA_BRAINTREE_PM.submit_payment_button.prop( 'disabled', false ).removeClass( 'spco-disabled-submit-btn' );
			var currency_code = eei18n.currency.code;
			braintree.setup( braintree_transaction_args.clientToken, 'dropin', {
				container : EEA_BRAINTREE_PM.dropin_container,
				form : EEA_BRAINTREE_PM.spco_payment_form,
				paypal : {
					singleUse : true,
					amount : parseFloat( EEA_BRAINTREE_PM.txn_data['payment_amount'] ),
					currency : currency_code
				},
				onReady : function( integration ) {
					EEA_BRAINTREE_PM.checkout = integration;
				},
				onError : function( error_data ) {
					if ( typeof error_data.message !== 'undefined' ) {
						EEA_BRAINTREE_PM.display_error( error_data.type, error_data.message, true );
					}
					// Reset PM drop-in.
					EEA_BRAINTREE_PM.reset_bt_dropin();
				},
				onPaymentMethodReceived : function( nonce_data ) {
					if ( typeof nonce_data.nonce !== 'undefined' && nonce_data.nonce ) {
						EEA_BRAINTREE_PM.pm_nonce.val( 'nonce' );
						EEA_BRAINTREE_PM.pm_nonce.val( nonce_data.nonce );
						EEA_BRAINTREE_PM.checkout_success( nonce_data.details, nonce_data.type );
					} else {
						EEA_BRAINTREE_PM.display_error( nonce_data.details, nonce_data.type );
					}
					// Reset PM drop-in.
					EEA_BRAINTREE_PM.reset_bt_dropin();
				}
			});
		},


		/**
		 * @function reset_bt_dropin
		 */
		reset_bt_dropin : function() {
			// Reset the drop-in.
			EEA_BRAINTREE_PM.checkout.teardown( function () {
				EEA_BRAINTREE_PM.checkout = null;
				EEA_BRAINTREE_PM.setup_braintree();
			});
		},


		/**
		 * @function checkout_success
		 * @param  {object} details
		 * @param  {string} type
		 */
		checkout_success : function( details, type ) {
			// Enable SPCO submit buttons.
			SPCO.enable_submit_buttons();
			if ( typeof details !== 'undefined' ) {
				EEA_BRAINTREE_PM.submit_payment_button.prop( 'disabled', true ).addClass( 'spco-disabled-submit-btn' );
				SPCO.offset_from_top_modifier = EEA_BRAINTREE_PM.offset_from_top_modifier;
				// Hide any return to cart buttons, etc.
				$( '.hide-me-after-successful-payment-js' ).hide();
				// Trigger click event on SPCO "Proceed to Next Step" button.
				EEA_BRAINTREE_PM.submit_payment_button.parents( 'form:first' ).find( '.spco-next-step-btn' ).trigger( 'click' );
			}
		},


		/**
		 * @function set_listener_for_payment_notice
		 */
		set_listener_for_payment_notice : function() {
			SPCO.main_container.on( 'spco_process_response', function( next_step, response ) {
				EEA_BRAINTREE_PM.submit_payment_button.prop( 'disabled', false ).removeClass( 'spco-disabled-submit-btn' );
				SPCO.allow_enable_submit_buttons = false;
				SPCO.disable_submit_buttons();
			});
		},


		/**
		 * @function set_listener_for_payment_method_selector
		 */
		set_listener_for_payment_method_selector : function() {
			SPCO.main_container.on( 'click', '.spco-next-step-btn', function() {
				EEA_BRAINTREE_PM.disable_SPCO_submit_buttons_if_Braintree_selected();
			});
		},


		/**
		 * @function disable_SPCO_submit_buttons_if_Braintree_selected
		 * Deactivate SPCO submit buttons to prevent submitting with no PM Nonce.
		 */
		disable_SPCO_submit_buttons_if_Braintree_selected : function() {
			if (
				EEA_BRAINTREE_PM.selected
				&& EEA_BRAINTREE_PM.submit_payment_button.length > 0
				&& EEA_BRAINTREE_PM.submit_payment_button.val().length <= 0
				&& EEA_BRAINTREE_PM.pm_nonce.length > 0
				&& EEA_BRAINTREE_PM.pm_nonce.val().length <= 0
			) {
				SPCO.allow_enable_submit_buttons = false;
				SPCO.disable_submit_buttons();
			}
		},


		/**
		 * @function hide_braintree
		 */
		hide_braintree : function() {
			EEA_BRAINTREE_PM.payment_method_selector.hide();
			EEA_BRAINTREE_PM.payment_method_info_div.hide();
		},


		/**
		 * @function display_error
		 * @param  {string} message
		 * @param  {string} type
		 * @param  {boolean} next_step_disabled
		 */
		display_error : function( type, message, next_step_disabled ) {
			SPCO.end_ajax();
			// Center notices on screen.
			$('#espresso-ajax-notices').eeCenter( 'fixed' );
			// Target parent container.
			var braintree_ajax_msg = $('#espresso-ajax-notices-error');
			// Actual message container.
			braintree_ajax_msg.children('.espresso-notices-msg').html( message );
			// Bye bye spinner.
			$('#espresso-ajax-loading').fadeOut('fast');
			// Display message.
			braintree_ajax_msg.removeClass('hidden').show().delay(10000).fadeOut('slow');

			// Log the Error.
			$.ajax({
				type : 'POST',
				dataType : 'json',
				url : eei18n.ajax_url,
				data : {
					action : 'eea_braintree_log_error',
					txn_id : EEA_BRAINTREE_PM.txn_data['TXN_ID'],
					message : message
				}
			});
			// Reset.
			if ( ! next_step_disabled ) {
				SPCO.enable_submit_buttons();
				SPCO.allow_submit_reg_form = true;
			}
		}
	};
	// End of EEA_BRAINTREE_PM object.



	// Initialize Braintree Dropin if the SPCO reg step changes to "payment_options".
	SPCO.main_container.on( 'spco_display_step', function( event, step_to_show ) {
		if ( typeof step_to_show !== 'undefined' && step_to_show === 'payment_options' ) {
			EEA_BRAINTREE_PM.initialize();
		}
	});

	// Initialize Braintree Dropin if the selected method of payment changes.
	SPCO.main_container.on( 'spco_switch_payment_methods', function( event, payment_method ) {
		//SPCO.console_log( 'payment_method', payment_method, false );
		if ( typeof payment_method !== 'undefined' && payment_method === braintree_transaction_args.payment_method_slug ) {
			EEA_BRAINTREE_PM.selected = true;
			EEA_BRAINTREE_PM.initialize();
		} else {
			EEA_BRAINTREE_PM.selected = false;
		}
	});

	// If loaded oad on the "payment_options" step with Braintree already selected.
	EEA_BRAINTREE_PM.initialize();
});