<?php if ( ! defined( 'EVENT_ESPRESSO_VERSION' ) ) { exit( 'No Direct script allowed.' ); }


/**
 * Class EE_MIGS_Payment_Method
 *
 * @package			Event Espresso
 * @subpackage		eea-migs-payment-method
 * @author			Event Espresso
 * @version			$VID:$
 */
class EE_MIGS_Payment_Method extends EE_Addon {


	/**
	 *	Register MIGS Gateway add-on.
	 *
	 *	@access public
	 *	@return void
	 */
	public static function register_addon() {
		// Register this PM Add-on vis a Plugin API.
		EE_Register_Addon::register(
			'MIGS_Payment_Method',
			array(
				'version' => EEA_MIGS_PM_VERSION,
				'min_core_version' => '4.8.0.rc.000',
				'main_file_path' => EEA_MIGS_PM_PLUGIN_FILE,
				'admin_callback' => 'additional_migs_mc_admin_hooks',
				'pue_options' => array(
					'pue_plugin_slug' => 'eea-migs-payment-method',
					'plugin_basename' => EEA_MIGS_PM_PLUGIN_BNAME,
					'checkPeriod' => '24',
					'use_wp_update' => false
				),
				// Register autoloaders.
				'autoloader_paths' => array(
					'EE_PMT_Base' => EE_LIBRARIES . 'payment_methods' . DS . 'EE_PMT_Base.lib.php',
					'EE_PMT_MIGS_Onsite' => EEA_MIGS_PM_PLUGIN_DIR . 'payment_methods' . DS . 'MIGS_Onsite' . DS . 'EE_PMT_MIGS_Onsite.pm.php',
				),
				'payment_method_paths' => array(
					EEA_MIGS_PM_PLUGIN_DIR . 'payment_methods' . DS . 'MIGS_Onsite'
				)
			)
		);
	}


	/**
	 *	Update the currencies supported by this gateway (if changed).
	 *
	 *	@access public
	 *	@return void
	 */
	public function initialize_default_data() {
		parent::initialize_default_data();
		$migs = EEM_Payment_method::instance()->get_one_of_type( 'MIGS_Onsite' );
		// Update If the payment method already exists.
		if ( $migs ) {
			$currencies = $migs->get_all_usable_currencies();
			$all_related = $migs->get_many_related( 'Currency' );
			if ( ( $currencies != $all_related ) ) {
				$migs->_remove_relations( 'Currency' );
				foreach ( $currencies as $currency_obj ) {
					$migs->_add_relation_to( $currency_obj, 'Currency' );
				}
			}
		}
	}


	/**
	 *	Additional admin hooks.
	 *
	 *	@access public
	 *	@return void
	 */
	public static function additional_migs_mc_admin_hooks() {
		// Is admin and not in M-Mode ?
		if ( is_admin() && ! EE_Maintenance_Mode::instance()->level() ) {
			add_filter( 'plugin_action_links', array( 'EE_MIGS_Payment_Method', 'plugin_actions' ), 10, 2 );
		}
	}


	/**
	 *	Add a settings link to the Plugins page.
	 *
	 *	@param $links
	 *	@param $file
	 *	@return array
	 */
	public static function plugin_actions( $links, $file ) {
		if ( $file == EEA_MIGS_PM_PLUGIN_BNAME ) {
			array_unshift( $links, '<a href="admin.php?page=espresso_payment_settings">' . __('Settings') . '</a>' );
		}
		return $links;
	}
}
// End of class EE_MIGS_Payment_Method.
// End of file EE_MIGS_Payment_Method.class.php