<?php if ( ! defined( 'EVENT_ESPRESSO_VERSION' ) ) { exit( 'No Direct script allowed.' ); }


/**
 * Class EEG_MIGS_Onsite
 *
 * @package			Event Espresso
 * @subpackage		eea-migs-payment-method
 * @author			Event Espresso
 * @version			$VID:$
 */
class EEG_MIGS_Onsite extends EE_Onsite_Gateway {

	/**
	 *	MIGS API user ID.
	 *	@var string
	 */
	protected $_merchant_id = null;


	/**
	 *	MIGS API user access code.
	 *	@var string
	 */
	protected $_access_code = null;


	/**
	 *	Currencies supported by this gateway.
	 *	@var array
	 */
	protected $_currencies_supported = array( 'AUD' );


	/**
	 *	Possible gateway error responses.
	 *	@var array
	 */
	protected $_migs_error = array();


	/**
	 *
	 *	@param EEI_Payment  $payment
	 *	@param array        $billing_info
	 *	@return \EE_Payment|\EEI_Payment
	 */
	public function do_direct_payment( $payment, $billing_info = null ) {
		$this->_migs_error = json_decode( file_get_contents(EEA_MIGS_PM_PLUGIN_DIR . 'migs-response-code-mapping.json'), true );
		$dbg_prefix = ( $this->_debug_mode ) ? 'TEST' : '';
		$merchant_id = $dbg_prefix . $this->_merchant_id;
		$request_url = 'https://migs.mastercard.com.au/vpcdps';

		$transaction = $payment->transaction();
		$primary_registrant = $transaction->primary_registration();

		// Get the request data ready.
		$transaction_details = array(
			'vpc_Version' => '1',
			'vpc_Command' => 'pay',

			'vpc_MerchTxnRef' => substr( $dbg_prefix . date( 'YmdHis' ) . '-' . rand( 0, 99999 ), 0, 40 ),
			'vpc_AccessCode' => $this->_access_code,
			'vpc_Merchant' => $this->_merchant_id,

			'vpc_OrderInfo' => substr( sprintf( __('Payment of %2$s for %2$s', 'event_espresso'), $payment->amount(), $primary_registrant->reg_code() ), 0, 34 ),
			'vpc_Amount' => str_replace( array(',', '.'), '', number_format( $payment->amount(), 2) ),
			'vpc_CardNum' => $billing_info['credit_card'],
			'vpc_CardExp' => substr($billing_info['exp_year'], -2) . $billing_info['exp_month'],
			'vpc_CardSecurityCode' => $billing_info['cvv']
		);

		// Send post request to MIGS.
		$this->_log_clean_request($transaction_details, $payment);
		$request_response = wp_remote_post(
			$request_url,
			array(
				'method' => 'POST',
				'timeout' => 60,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking' => true,
				'headers' => array(),
				'body' => $transaction_details,
				'cookies' => array()
			)
		);
		// Do we get an OK response from MIGS?
		if ( isset($request_response['body']) && strpos($request_response['body'], 'vpc_TxnResponseCode') !== false ) {
			$r_respomse = explode('&', $request_response['body']);
			$response = array();
			foreach ( $r_respomse as $val ) {
				$cycler = array();
				$cycler = explode('=', $val);
				$response[$cycler[0]] = $cycler[1];
			}

			$this->_log_clean_response($response, $payment);
			$txn_sts = $response['vpc_TxnResponseCode'];
			// If present, error message and description.
			$e_message = $this->_migs_error[$txn_sts];
			$e_description = is_numeric($response['vpc_AcqResponseCode']) ? '('.$this->_migs_error[$response['vpc_AcqResponseCode']].')' : '';
			$vpc_Message = $e_message . ' ' . $e_description;
			// Now check the TXN status.
			if ( $txn_sts === '0' ) {
				$payment->set_status( $this->_pay_model->approved_status() );
				$payment->set_gateway_response( str_replace( '+', ' ', $response['vpc_Message'] ) );
				$payment->set_amount( floatval( $response['vpc_Amount'] / 100 ) );
			} elseif ( in_array($txn_sts, array('B', 'U', 'E', '2')) ) {
				$payment->set_status( $this->_pay_model->declined_status() );
				$payment->set_gateway_response( $vpc_Message );
			} else {
				$payment->set_status( $this->_pay_model->failed_status() );
				$payment->set_gateway_response( $vpc_Message );
			}
			$payment->set_details($response);
			$payment->set_txn_id_chq_nmbr( isset($response['vpc_TransactionNo']) ? $response['vpc_TransactionNo'] : null );
		} else {
			$payment->set_status( $this->_pay_model->failed_status() );
			$payment->set_gateway_response( $request_response );
		}

		return $payment;
	}


	/**
	 *
	 *	@param array        $response
	 *	@param EEI_Payment  $payment
	 *	@return void
	 */
	private function _log_clean_request( $request, $payment ) {
		$cleaned_request_data = $request;
		unset($cleaned_request_data['vpc_CardSecurityCode']);
		unset($cleaned_request_data['vpc_CardExp']);
		unset($cleaned_request_data['vpc_CardNum']);
		$this->log( array('MIGS Request' => $cleaned_request_data), $payment );
	}


	/**
	 *
	 *	@param array        $response
	 *	@param EEI_Payment  $payment
	 *	@return void
	 */
	private function _log_clean_response( $response, $payment ) {
		$cleaned_response_data = $response;
		unset($cleaned_response_data['vpc_CSCResultCode']);
		unset($cleaned_response_data['vpc_CSCRequestCode']);
		$this->log( array('MIGS Response' => $cleaned_response_data), $payment );
	}

}
// End of class EEG_MIGS_Onsite.
// End of file EEG_MIGS_Onsite.pm.php