<?php if ( ! defined( 'EVENT_ESPRESSO_VERSION' ) ) { exit( 'No Direct script allowed.' ); }


/**
 * Class EE_PMT_MIGS_Onsite
 *
 * @package			Event Espresso
 * @subpackage		eea-migs-payment-method
 * @author			Event Espresso
 * @version			$VID:$
 */
class EE_PMT_MIGS_Onsite extends EE_PMT_Base {

	/**
	 *	Path to the templates folder.
	 *	@var string
	 */
	protected $_template_path = null;


	/**
	 *	Class constructor.
	 *
	 *	@param EE_Payment_Method $pm_instance
	 *	@throws \EE_Error
	 *	@return \EEA_MIGS_Payment_Method
	 */
	public function __construct( $pm_instance = null ) {
		$this->_pretty_name = __("MIGS", 'event_espresso');
		$this->_default_description = __( 'Fill up the form below and then Click the "Finalize Registration" button to proceed with the payment.', 'event_espresso' );
		$this->_template_path = dirname( __FILE__ ) . DS . 'templates' . DS;
		$this->_default_button_url = EEA_MIGS_PM_PLUGIN_URL . 'payment_methods' . DS . 'MIGS_Onsite' . DS . 'lib' . DS . 'migs-onsite-gateway.png';
		require_once( $this->file_folder() . 'EEG_MIGS_Onsite.gateway.php' );
		$this->_gateway = new EEG_MIGS_Onsite();

		parent::__construct( $pm_instance );

		// MIGS styles.
		add_action('wp_enqueue_scripts', array($this, 'load_styles'));
	}


	/**
	 *	Generate a new payment settings form.
	 *
	 *	@return EE_Payment_Method_Form
	 */
	public function generate_new_settings_form() {
		return new EE_Payment_Method_Form( array(
			'extra_meta_inputs' => array(
				'merchant_id' => new EE_Text_Input( array(
					'html_label_text' => sprintf( __("MIGS Merchant ID %s", "event_espresso"), $this->get_help_tab_link() ),
					'required' => true
				)),
				'access_code' => new EE_Password_Input( array(
					'html_label_text' => sprintf( __("MIGS Access Code %s", "event_espresso"), $this->get_help_tab_link() ),
					'required' => true
				))
			)
		));
	}


	/**
	 *	Creates a billing form for this payment method type.
	 *
	 *	@param \EE_Transaction $transaction
	 *	@return \EE_Billing_Info_Form
	 */
	public function generate_new_billing_form( EE_Transaction $transaction = null ) {
		require_once( $this->file_folder() . 'EEA_MIGS_Billing_Form.form.php' );
		$form =  new EEA_MIGS_Billing_Form( $this, $this->_pm_instance );
		return $this->apply_billing_form_debug_settings( $form );
	}


	/**
	 *	Possibly adds debug content to MIGS checkout billing form.
	 *
	 *	@param \EE_Billing_Info_Form $billing_form
	 *	@return \EE_Billing_Info_Form
	 */
	public function apply_billing_form_debug_settings( EE_Billing_Info_Form $billing_form ) {
		if ( $this->_pm_instance->debug_mode() ) {
			$billing_form->add_subsections(
				array( 'fyi_about_autofill' => $billing_form->payment_fields_autofilled_notice_html() ),
				'account_type'
			);
			$billing_form->add_subsections(
				array( 'debug_content' => new EE_Form_Section_HTML_From_Template( $this->_template_path . 'migs_debug_info.template.php' ) ),
				'first_name'
			);
			$billing_form->get_input( 'credit_card' )->set_default( '5123456789012346' );
			$billing_form->get_input( 'exp_month' )->set_default( '05' );
			$billing_form->get_input( 'exp_year' )->set_default( '2017' );
			$billing_form->get_input( 'cvv' )->set_default( '123' );
		}
		return $billing_form;
	}


	/**
	 *	Adds the help tab.
	 *
	 *	@see EE_PMT_Base::help_tabs_config()
	 *	@return array
	 */
	public function help_tabs_config() {
		return array(
			$this->get_help_tab_name() => array(
				'title' => __( 'MIGS Settings', 'event_espresso' ),
				'filename' => 'payment_methods_overview_migs'
			)
		);
	}


	/**
	 *  Load MIGS gateway Styles.
	 *
	 *	@return void
	 */
	public function load_styles() {
		wp_enqueue_style( 'migs-base-stylesheet', EEA_MIGS_PM_PLUGIN_URL . 'assets' . DS . 'css' . DS . 'migs-base-styles.css', array(), EEA_MIGS_PM_VERSION );
	}
}
// End of class EE_PMT_MIGS_Onsite.
// End of file EE_PMT_MIGS_Onsite.pm.php