<?php if ( ! defined('EVENT_ESPRESSO_VERSION') ) { exit('NO direct script access allowed'); }

define('EEA_PAYEEZY_PM_PLUGIN_BASENAME', plugin_basename(EEA_PAYEEZY_PM_PLUGIN_FILE));
define('EEA_PAYEEZY_PM_PLUGIN_URL', plugin_dir_url(EEA_PAYEEZY_PM_PLUGIN_FILE));

/**
 * ----------------------------------------------
 *
 * Class  EE_Payeezy
 *
 * @package			Event Espresso
 * @subpackage		eea-first-data-payeezy
 * @author			Event Espresso
 * @version		 	$VID:$
 *
 * ----------------------------------------------
 */
class EE_Payeezy extends EE_Addon {

	/**
	 *	Register this add-on with EE.
	 *
	 *	@return void
	 */
	public static function register_addon() {
		// Register add-on via EE Plugin API.
		EE_Register_Addon::register(
			'Payeezy',
			array(
				'version' => EEA_PAYEEZY_PM_VERSION,
				'min_core_version' => '4.9.17.dev.000',
				'main_file_path' => EEA_PAYEEZY_PM_PLUGIN_FILE,
				'admin_callback' => 'additional_admin_hooks',
				'payment_method_paths' => array(
					EEA_PAYEEZY_PM_PLUGIN_PATH . 'payment_methods' . DS . 'Payeezy_Onsite'
				),
				// Register auto-loaders.
				'autoloader_paths' => array(),
				// If plugin update engine is being used for auto-updates. not needed if PUE is not being used.
				'pue_options' => array(
					'pue_plugin_slug' => 'eea-payeezy',
					'plugin_basename' => EEA_PAYEEZY_PM_PLUGIN_BASENAME,
					'checkPeriod' => '24',
					'use_wp_update' => FALSE
				),
		));
	}


	/**
	 * 	Additional admin hooks.
	 *
	 *  @access public
	 *  @return void
	 */
	public static function additional_admin_hooks() {
		// Is admin and not in M-Mode ?
		if ( is_admin() && ! EE_Maintenance_Mode::instance()->level() ) {
			add_filter('plugin_action_links', array('EE_Payeezy', 'plugin_actions'), 10, 2);
		}
	}


	/**
	 * Add a settings link to the Plugins page, so people can go straight from the plugin page to the settings page.
	 *
	 * @param $links
	 * @param $file
	 * @return array
	 */
	public static function plugin_actions( $links, $file ) {
		if ( $file == EEA_PAYEEZY_PM_PLUGIN_BASENAME ) {
			// Before other links.
			array_unshift($links, '<a href="admin.php?page=espresso_payment_settings">' . __('Settings') . '</a>');
		}
		return $links;
	}


	/**
	 * 	Setup default data for the add-on.
	 *
	 *  @access public
	 *  @return void
	 */
	public function initialize_default_data() {
		parent::initialize_default_data();

		// Update the currencies supported by this gateway (if changed).
		$payeezy = EEM_Payment_method::instance()->get_one_of_type( 'Payeezy_Onsite' );
		// Update If the payment method already exists.
		if ( $payeezy ) {
			$currencies = $payeezy->get_all_usable_currencies();
			$all_related = $payeezy->get_many_related( 'Currency' );

			if ( ($currencies != $all_related) ) {
				$payeezy->_remove_relations( 'Currency' );
				foreach ( $currencies as $currency_obj ) {
					$payeezy->_add_relation_to( $currency_obj, 'Currency' );
				}
			}
		}
	}
}
// End of file EE_Payeezy.class.php
// Location: wp-content/plugins/espresso-payeezy/