<?php

if (!defined('EVENT_ESPRESSO_VERSION')) {
	exit('No direct script access allowed');
}

/**
 *
 * EEG_SagePay_Onsite
 *
 * Very similar to EE4 core's Paypal Pro payment method
 * @see ..{EE4-core}\payment_methods\Paypal_Pro\EEG_Paypal_Pro.gateway.php - for more details on each method.
 *
 * @package			Event Espresso
 * @subpackage		eea-sage-pay-gateway
 * @author			Event Espresso
 *
 */
class EEG_Sage_Pay_Onsite extends EE_Onsite_Gateway {

	/**
	 *
	 * @var $_sage_pay_vendor_name string.
	 */
	protected $_vendor_name = null;


	protected $_credit_card_types = null;


	protected $_currencies_supported = array(
		'GBP',
		'EUR',
		'USD'
	);


	public function do_direct_payment($payment, $billing_info = null) {
		$transaction = $payment->transaction();
		$primary_registrant = $transaction->primary_registration();

		// Get the request data ready.
		$transaction_details = array(
			'VPSProtocol' => '3.00',
			'TxType' => 'PAYMENT',
			'Vendor' => $this->_vendor_name,
			'VendorTxCode' => $primary_registrant->reg_code(),
			'Amount' => $this->format_currency($payment->amount()),
			'Currency' => $payment->currency_code(),
			'Description' => sprintf( __("Payment of %s for %s", "event_espresso"), $payment->amount(), $primary_registrant->reg_code() ),

			'CardHolder' => $billing_info['card_holder'],
			'CardNumber' => $billing_info['credit_card'],
			'ExpiryDate' => $billing_info['exp_month'] . substr($billing_info['exp_year'], -2),
			'CV2' => $billing_info['cvv'],
			'CardType' => $billing_info['credit_card_type'],

			'CustomerEmail' => $billing_info['email'],

			'BillingSurname' => $billing_info['last_name'],
			'BillingFirstnames' => $billing_info['first_name'],
			'BillingAddress1' => $billing_info['address'],
			'BillingAddress2' => $billing_info['address2'],
			'BillingCity' => $billing_info['city'],
			'BillingPostCode' => $billing_info['zip'],
			'BillingCountry' => $billing_info['country'],
			'BillingPhone' => substr($billing_info['phone'], 0, 20),

			'DeliverySurname' => $billing_info['last_name'],
			'DeliveryFirstnames' => $billing_info['first_name'],
			'DeliveryAddress1' => $billing_info['address'],
			'DeliveryCity' => $billing_info['city'],
			'DeliveryPostCode' => $billing_info['zip'],
			'DeliveryCountry' => $billing_info['country']
		);

		// The BillingState and DeliveryState becomes mandatory when the BillingCountry is set to US.
		if ( $billing_info['country'] === 'US' || $billing_info['country'] === 'CA' ) {
			$transaction_details['BillingState'] = $billing_info['state'];
			$transaction_details['DeliveryState'] = $billing_info['state'];
		}

		// Send the payment Request.
		$request_response = $this->_sage_send_request( $transaction_details, $payment );

		$sagePayResponse = array();
		if ( is_string($request_response) ) {
			$response_args = preg_split("/[\r\n]+/", $request_response);
			foreach ($response_args as $resp_value) {
				$interm_val = explode('=', $resp_value);
				$sagePayResponse[$interm_val[0]] = $interm_val[1];
			}
		} elseif( is_array($request_response) ) {
			$sagePayResponse = $request_response;
		}

		$this->_log_clean_response($sagePayResponse, $payment);

		if ( isset($sagePayResponse['Status']) ) {
			if ( ($sagePayResponse['Status'] === 'OK') && isset($sagePayResponse['TxAuthNo']) ) {
				$payment->set_status( $this->_pay_model->approved_status() );
				$payment->set_gateway_response($sagePayResponse['StatusDetail']);
			} elseif ( $sagePayResponse['Status'] === 'NOTAUTHED' ) {
				$payment->set_status( $this->_pay_model->declined_status() );
				$payment->set_gateway_response('Your payment was declined. Please try a different Card.');
			} elseif ( $sagePayResponse['Status'] === 'REJECTED' ) {
				$payment->set_status( $this->_pay_model->declined_status() );
				$payment->set_gateway_response('Your payment was rejected. Please check that your Address and  CV2 details are correct.');
			} elseif ( $sagePayResponse['Status'] === 'MALFORMED' || $sagePayResponse['Status'] === 'INVALID' ) {
				$payment->set_status( $this->_pay_model->declined_status() );
				$payment->set_gateway_response($sagePayResponse['StatusDetail']);
			} elseif ( $sagePayResponse['Status'] === 'ERROR' ) {
				$payment->set_status( $this->_pay_model->failed_status() );
				$payment->set_gateway_response($sagePayResponse['StatusDetail']);
			}
			$payment->set_details($sagePayResponse);
			$payment->set_txn_id_chq_nmbr( isset($sagePayResponse['TxAuthNo']) ? $sagePayResponse['TxAuthNo'] : null );
			return $payment;
		} else {
			$payment->set_status( $this->_pay_model->failed_status() );
			$payment->set_gateway_response($sagePayResponse['StatusDetail']);
			return $payment;
		}
	}


	/**
	 * Posts a request to Sage Pay and returns the response.
	 *
	 * @return array  The response from Sage Pay.
	 */
	private function _sage_send_request( $transaction_data, $payment ) {
		$post_fields = array();
		$this->_log_clean_request($transaction_data, $payment);

		foreach ( $transaction_data as $key => $value ) {
			$post_fields[] = $key . '=' . urlencode($value);
		}
		$post_url = ( $this->_debug_mode ) ? 'https://test.sagepay.com/gateway/service/vspdirect-register.vsp' : 'https://live.sagepay.com/gateway/service/vspdirect-register.vsp';
		$response = wp_remote_post(
			$post_url,
			array(
				'method' => 'POST',
				'timeout' => 60,
				'redirection' => 5,
				'httpversion' => '1.0',
				'blocking' => true,
				'headers' => array(),
				'body' => $transaction_data,
				'cookies' => array()
			)
		);

		return $response['body'];
	}


	private function _log_clean_request( $request, $payment ) {
		$cleaned_request_data = $request;
		unset($cleaned_request_data['CV2']);
		unset($cleaned_request_data['ExpiryDate']);
		unset($cleaned_request_data['CardNumber']);
		unset($cleaned_request_data['CardHolder']);
		$this->log( array('EE SagePay Request' => $cleaned_request_data), $payment );
	}


	private function _log_clean_response( $response, $payment ) {
		$cleaned_response_data = $response;
		unset($cleaned_response_data['ExpiryDate']);
		unset($cleaned_response_data['BankAuthCode']);
		$this->log( array('EE SagePay Response' => $cleaned_response_data), $payment );
	}
}

// End of file EEG_Sage_Pay_Onsite.gateway.php
