<?php

if (!defined('EVENT_ESPRESSO_VERSION')) {
	exit('No direct script access allowed');
}

/**
 *
 * EE_PMT_Onsite
 * Very similar to EE4 core's Paypal Pro payment method
 * @see ..{EE4-core}\payment_methods\Paypal_Pro\EE_PMT_Paypal_Pro.pm.php - for more details on each method.
 *
 *
 * @package			Event Espresso
 * @subpackage		eea-sage-pay-gateway
 * @author			Event Espresso
 *
 */

class EE_PMT_Sage_Pay_Onsite extends EE_PMT_Base {

	/**
	 * path to the templates folder for the Sage Pay PM
	 * @var string
	 */
	protected $_template_path = null;


	/**
	 * @return \EE_PMT_Sage_Pay_Onsite
	 */
	public function __construct( $pm_instance = null ) {
		require_once( $this->file_folder() . 'EEG_Sage_Pay_Onsite.gateway.php' );
		$this->_gateway = new EEG_Sage_Pay_Onsite();
		$this->_pretty_name = __("Sage Pay", 'event_espresso');
		$this->_default_description = __( 'Please provide the following billing information.', 'event_espresso' );
		$this->_template_path = dirname(__FILE__) . DS . 'templates' . DS;
		$this->_requires_https = true;

		parent::__construct( $pm_instance );
		$this->_default_button_url = $this->file_url() . 'lib' . DS . 'sagepay-default-logo.png';	// More logos can be found at: [http://www.sagepay.co.uk/logo-download-centre]
	}


	public function generate_new_settings_form() {

		return new EE_Payment_Method_Form( array(
			'extra_meta_inputs' => array(
				'vendor_name' => new EE_Text_Input( array(
					'html_label_text' => sprintf( __("Sage Pay Vendor Name %s", "event_espresso"), $this->get_help_tab_link() ),
					'required' => true
				)),
				'credit_card_types' => new EE_Checkbox_Multi_Input(
					$this->card_types_supported(),
					array(
						'required' => true,//they need to have at least one card active right?
						'default' => array(
							'VISA',
							'MC'
						)
					)
				),
			)
		));
	}


	public function generate_new_billing_form( EE_Transaction $transaction = null ) {
		$allowed_types = $this->_pm_instance->get_extra_meta( 'credit_card_types', true );
		// In case allowed types is a string or empty array or null.
		if ( ! $allowed_types ) {
			$allowed_types = array();
		}

		$form_name = 'Sage_Pay_Billing_Form';
		$form_args = array(
			'name' => $form_name,
			'html_id' => 'ee-Sage_Pay-billing-form',
			'subsections' => array(
				'credit_card' => new EE_Credit_Card_Input( 
					array( 
						'html_class' => 'ee-billing-qstn ee-sage-pay-billing-form-credit-card',
						'required' => true, 
						'html_label_text' => __('Card Number', 'event_espresso'),
					)
				),
				'credit_card_type' => new EE_Select_Input( 
					// The options are set dynamically
					array_intersect_key( 
						EE_PMT_Sage_Pay_Onsite::card_types_supported(), 
						array_flip( $allowed_types )
					), 
					array( 
						'html_class' => 'ee-billing-qstn ee-sage-pay-billing-form-card-type', 
						'required' => true, 
						'html_label_text' => __( 'Card Type', 'event_espresso' )
					)
				),	
				'exp_month' => new EE_Credit_Card_Month_Input( 
					true, 
					array( 
						'html_class' => 'ee-billing-qstn ee-sage-pay-billing-form-exp-month',
						'required' => true, 
						'html_label_text' => __( 'Expiry Month', 'event_espresso' ),
					)
				),
				'exp_year' => new EE_Credit_Card_Year_Input( 
					array( 
						'html_class' => 'ee-billing-qstn ee-sage-pay-billing-form-exp-year',
						'required' => true, 
						'html_label_text' => __( 'Expiry Year', 'event_espresso' ),
					)
				),
				'cvv' => new EE_CVV_Input( 
					array( 
						'html_class' => 'ee-billing-qstn ee-sage-pay-billing-form-card-cvv',
						'required' => true, 
						'html_label_text' => __( 'CVV', 'event_espresso' ),
				
					)
				),
				'card_holder' => new EE_Text_Input( 
					array(
						'html_class' => 'ee-billing-qstn ee-sage-pay-billing-form-card-name',
						'required' => true,
						'html_label_text' => __('Card Holder (name displayed on the card)', 'event_espresso'),
					)
				),
			)
		);
		$billing_form = new EE_Billing_Attendee_Info_Form( $this->_pm_instance, $form_args );

		return $this->apply_billing_form_debug_settings( $billing_form );
	}


	public function apply_billing_form_debug_settings( EE_Billing_Info_Form $billing_form ) {
		if ( $this->_pm_instance->debug_mode() ) {
			$billing_form->add_subsections(
				array( 'fyi_about_autofill' => $billing_form->payment_fields_autofilled_notice_html() ),
				'credit_card'
			);
			$billing_form->add_subsections(
				array( 'debug_content' => new EE_Form_Section_HTML_From_Template( dirname(__FILE__) . DS . 'templates' . DS . 'sage_pay_onsite_debug_info.template.php') ),
				'first_name'
			);
			$billing_form->get_input( 'credit_card' )->set_default( '4929000000006' );
			$billing_form->get_input( 'credit_card_type' )->set_default( 'VISA' );
			$billing_form->get_input( 'exp_year' )->set_default( date('Y') + 6 );
			$billing_form->get_input( 'cvv' )->set_default( '123' );
		}

		return $billing_form;
	}


	public static function card_types_supported() {
		return array(
			'VISA' => __("Visa", 'event_espresso'),
			'MC' => __("MasterCard", 'event_espresso'),
			'MCDEBIT' => __("Debit MasterCard", 'event_espresso'),
			'DELTA' => __("Visa Debit / Delta", 'event_espresso'),
			'MAESTRO' => __("Domestic and International issued Maestro", 'event_espresso'),
			'UKE' => __("Visa Electron", 'event_espresso'),
			'AMEX' => __("American Express", 'event_espresso'),
			'DC' => __("Diners Club International and Discover", 'event_espresso'),
			'JCB' => __("Japan Credit Bureau", 'event_espresso'),
			'LASER' => __("Laser", 'event_espresso')
		);
	}


	public function help_tabs_config(){
		//determine the PUBLIC ip address via this 3rd party service
		//but don't be too trusting of it either
		$ip = strip_tags( wp_remote_retrieve_body( wp_remote_get( 'https://icanhazip.com' ) ) );
		return array(
			$this->get_help_tab_name() => array(
				'title' => __('Sage Pay Settings', 'event_espresso'),
				'filename' => 'payment_methods_overview_sage_pay',
				'template_args' => array(
					'ip' => $ip
				)
			),
		);
	}


	protected function _get_billing_values_from_form( $billing_form ) {
		$billing_values = parent::_get_billing_values_from_form( $billing_form );
		$billing_values[ 'country' ] = $billing_form->get_input_value( 'country' );
		$billing_values[ 'credit_card_type' ] = $billing_form->get_input_value( 'credit_card_type' );

		$state = EEM_State::instance()->get_col( array( array('STA_name' => $billing_values['state']), 'limit' => 1), 'STA_abbrev' );
		$billing_values[ 'state' ] = $state[0];
		return $billing_values;
	}
}

// End of file EE_PMT_Sage_Pay_Onsite.pm.php
