<?php

if (!defined('EVENT_ESPRESSO_VERSION')) {
    exit('NO direct script access allowed');
}

define('EEA_VANCO_PM_BASENAME', plugin_basename(EEA_VANCO_PM_PLUGIN_FILE));
define('EEA_VANCO_PM_PATH', plugin_dir_path(EEA_VANCO_PM_PLUGIN_FILE));
define('EEA_VANCO_PM_URL', plugin_dir_url(EEA_VANCO_PM_PLUGIN_FILE));



/**
 * Class EE_Vanco_Payment_Method
 *
 * @package			Event Espresso
 * @subpackage		eea-vanco-payment-method
 * @author			Event Espresso
 *
 */
class EE_Vanco_Payment_Method extends EE_Addon
{


    /**
     *    Register this add-on.
     *
     * @access public
     * @return void
     * @throws \EE_Error
     */
    public static function register_addon()
    {
        // Register this PM Add-on vis a Plugin API.
        if (extension_loaded('openssl')) {
            EE_Register_Addon::register(
                'Vanco_Payment_Method',
                array(
                    'version'              => EEA_VANCO_PM_VERSION,
                    'min_core_version'     => '4.9.26.rc.000',
                    'main_file_path'       => EEA_VANCO_PM_PLUGIN_FILE,
                    'admin_callback'       => 'additional_vanco_admin_hooks',
                    // register autoloaders
                    'autoloader_paths'     => array(
                        'EE_PMT_Base'         => EE_LIBRARIES . 'payment_methods' . DS . 'EE_PMT_Base.lib.php',
                        'EE_PMT_Vanco_Onsite' => EEA_VANCO_PM_PATH . 'payment_methods' . DS . 'Vanco_Onsite'
                                                    . DS . 'EE_PMT_Vanco_Onsite.pm.php',
                    ),
                    // If plugin update engine is being used for auto-updates.
                    'pue_options'          => array(
                        'pue_plugin_slug' => 'eea-vanco-payment-method',
                        'plugin_basename' => EEA_VANCO_PM_BASENAME,
                        'checkPeriod'     => '24',
                        'use_wp_update'   => false,
                    ),
                    'payment_method_paths' => array(
                        EEA_VANCO_PM_PATH . 'payment_methods' . DS . 'Vanco_Onsite',
                    ),
                )
            );
        } else {
            //don't use EE_Error::add_error() because it currently has at least one bug.
            // See https://github.com/eventespresso/event-espresso-core/issues/589 and
            // https://github.com/eventespresso/event-espresso-core/issues/586
            add_action('admin_notices', array('EE_Vanco_Payment_Method', 'addonRequirementsNotice'));
        }
    }


    /**
     * a safe space for addons to add additional logic like setting hooks
     * that will run immediately after addon registration
     * making this a great place for code that needs to be "omnipresent"
     */
    public function after_registration()
    {
        // Delete scheduled events on deactivation.
        register_deactivation_hook(EEA_VANCO_PM_PLUGIN_FILE, array($this, 'clear_schedules_on_deactivate'));
        // Vanco encrypt/decrypt nvpvar Requests.
        add_action('wp_ajax_eea_vanco_encrypt_nvpvar', array('EE_PMT_Vanco_Onsite', 'ajax_encrypt_nvpvar'));
        add_action('wp_ajax_nopriv_eea_vanco_encrypt_nvpvar', array('EE_PMT_Vanco_Onsite', 'ajax_encrypt_nvpvar'));
        // Vanco Login request.
        add_action('wp_ajax_eea_vanco_login', array('EE_PMT_Vanco_Onsite', 'vanco_ajax_login_request'));
        add_action('wp_ajax_nopriv_eea_vanco_login', array('EE_PMT_Vanco_Onsite', 'vanco_ajax_login_request'));
        // Log Vanco JS errors.
        add_action('wp_ajax_eea_vanco_log_error', array('EE_PMT_Vanco_Onsite', 'vanco_log_error'));
        add_action('wp_ajax_nopriv_eea_vanco_log_error', array('EE_PMT_Vanco_Onsite', 'vanco_log_error'));
    }


    /**
     *    Register this add-on.
     *
     * @access public
     * @return void
     * @throws \EE_Error
     */
    public function initialize_default_data()
    {
        parent::initialize_default_data();
        // Update the currencies supported by this gateway (if changed).
        $vanco = EEM_Payment_method::instance()->get_one_of_type('Vanco_Onsite');
        // Update If the payment method already exists.
        if ($vanco) {
            $currencies = $vanco->get_all_usable_currencies();
            $all_related = $vanco->get_many_related('Currency');
            if ($currencies !== $all_related) {
                $vanco->_remove_relations('Currency');
                foreach ($currencies as $currency_obj) {
                    $vanco->_add_relation_to($currency_obj, 'Currency');
                }
            }
        }
    }


    /**
     * 	Additional admin hooks.
     *
     * 	@access public
     * 	@return void
     */
    public static function additional_vanco_admin_hooks()
    {
        // is admin and not in M-Mode ?
        if (is_admin() && !EE_Maintenance_Mode::instance()->level()) {
            add_filter('plugin_action_links', array('EE_Vanco_Payment_Method', 'plugin_actions'), 10, 2);
        }
    }


    /**
     * 	Add a settings link to the Plugins page.
     *
     * 	@param $links
     * 	@param $file
     * 	@return array
     */
    public static function plugin_actions($links, $file)
    {
        if ($file === EEA_VANCO_PM_BASENAME) {
            // Before other links
            array_unshift($links, '<a href="admin.php?page=espresso_payment_settings">' . __('Settings') . '</a>');
        }
        return $links;
    }


    /**
     * clear_schedules_on_deactivate
     * clears any cron jobs set for Vanco PM
     */
    public function clear_schedules_on_deactivate()
    {
        wp_clear_scheduled_hook('eea_vanco_session_id_update');
    }


    /**
     * Content of the add-on requirements admin notice.
     *
     * @access public
     * @return void
     */
    public static function addonRequirementsNotice()
    {
        echo '<div class="error"><p>'
             . sprintf(
                 esc_html(
                     _x(
                         // @codingStandardsIgnoreStart
                         'The %1$sEvent Espresso - Vanco Gateway%2$s plugin requires the %1$sOpenSSL PHP extension%2$s to work properly.%3$sIf you are unsure of how to install/enable that module please contact your host and have them install it for you, the Vanco payment method will be deactivated until this issue is resolved.',
                         'The Event Espresso - Vanco Gateway plugin requires the OpenSSL PHP extension to work properly. If you are unsure of how to install/enable that module please contact your host and have them install it for you, the Vanco payment method will be deactivated until this issue is resolved.',
                         // @codingStandardsIgnoreEnd
                         'event_espresso'
                     )
                 ),
                 '<strong>',
                 '</strong>',
                 '<br>'
             )
             . '</p></div>';
    }
}

// End of class EE_Vanco_Payment_Method.
// End of file EE_Vanco_Payment_Method.class.php