<?php

if (!defined('EVENT_ESPRESSO_VERSION')) {
    exit('No direct script access allowed');
}



/**
 *
 * Class EEA_Vanco_Billing_Form
 *
 *
 * @package			Event Espresso
 * @subpackage		eea-vanco-payment-method
 * @author			Event Espresso
 *
 */
class EEA_Vanco_Billing_Form extends EE_Billing_Attendee_Info_Form
{


    protected function _normalize($req_data)
    {
        parent::_normalize($req_data);
    }


    /**
     *
     * @param EE_PMT_Vanco_Onsite $payment_method_type
     * @param EE_Payment_Method $payment_method
     */
    public function __construct(EE_PMT_Vanco_Onsite $payment_method_type, EE_Payment_Method $payment_method)
    {
        // Some of the settings in the default 'EE_Credit_Card_Input' does not fit so let's create a new input, similar to the default one.
        require_once( EEA_VANCO_PM_PATH . 'payment_methods' . DS . 'Vanco_Onsite' . DS . 'EE_Vanco_Credit_Card_Input.input.php' );

        $state_list = array('' => array('' => ''));
        $state_list += $payment_method_type->get_state_abbrv_list();

        //EEH_Debug_Tools::printr( $payment_method, '$payment_method', __FILE__, __LINE__ );
        //EEH_Debug_Tools::printr( $payment_method->settings_array(), '$payment_method->settings_array()', __FILE__, __LINE__ );
        //EEH_Debug_Tools::printr( $payment_method_type, '$payment_method_type', __FILE__, __LINE__ );
        $account_types = ( $payment_method->get_extra_meta('is_debitcard_only', true) ) ? array('CC' => 'Credit or Debit Card') : $payment_method_type->_account_types;
        $settings = $payment_method->settings_array();
        $options_array = array(
            'name'        => 'eea_vanco_onsite_billing_form',
            'subsections' => array(
                'state'          => new EE_Select_Input($state_list, array(
                    'required' => true
                    )),
                'account_type'   => new EE_Select_Input($account_types,
                    array(
                    'html_class'      => 'ee-vanco-billing-account-type',
                    'html_label_text' => esc_html__('Account Type', 'event_espresso'),
                    'required'        => true
                    )),
                'name_on_card'   => new EE_Text_Input(array(
                    'html_class'            => 'ee-vanco-billing-form-name-on-card ee-required',
                    'html_label_text'       => sprintf(esc_html__('Name on Card %s', 'event_espresso'),
                        '<span class="ee-asterisk">*</span>'),
                    'required'              => false,
                    'html_other_attributes' => 'required=""'
                    )),
                'credit_card'    => new EE_Vanco_Credit_Card_Input(array(
                    'html_class'            => 'ee-vanco-billing-form-credit-card',
                    'html_label_text'       => sprintf(esc_html__('Card Number %s', 'event_espresso'),
                        '<span class="ee-asterisk">*</span>'),
                    'required'              => false,
                    'html_other_attributes' => 'required=""'
                    )),
                'bank_account'   => new EE_Text_Input(array(
                    'html_class'            => 'ee-vanco-billing-form-bank-account ee-required',
                    'html_label_text'       => sprintf(esc_html__('Bank Account number %s', 'event_espresso'),
                        '<span class="ee-asterisk">*</span>'),
                    'required'              => false,
                    'html_other_attributes' => 'required=""'
                    )),
                'routing_number' => new EE_Text_Input(array(
                    'html_class'            => 'ee-vanco-billing-form-routing-number ee-required',
                    'html_label_text'       => sprintf(esc_html__('Routing Number %s', 'event_espresso'),
                        '<span class="ee-asterisk">*</span>'),
                    'required'              => false,
                    'html_other_attributes' => 'required=""'
                    )),
                'exp_month'      => new EE_Credit_Card_Month_Input(true,
                    array(
                    'html_class'      => 'ee-vanco-billing-form-exp-month',
                    'html_label_text' => esc_html__('Expiry Month', 'event_espresso'),
                    'required'        => true
                    )),
                'exp_year'       => new EE_Credit_Card_Year_Input(array(
                    'html_class'      => 'ee-vanco-billing-form-exp-year',
                    'html_label_text' => esc_html__('Expiry Year', 'event_espresso'),
                    'required'        => true
                    )),
                'vanco_nvpvar'           => new EE_Hidden_Input(array(
                    'html_class'                      => 'eea-vanco-nvpvar',
                    'default'                         => false,
                    'sensitive_data_removal_strategy' => new EE_All_Sensitive_Data_Removal()
                    )),
                'vanco_sessionid'        => new EE_Hidden_Input(array(
                    'html_class'                      => 'eea-vanco-sessionid',
                    'default'                         => false,
                    'sensitive_data_removal_strategy' => new EE_All_Sensitive_Data_Removal()
                    )),
                'vanco_requestid'        => new EE_Hidden_Input(array(
                    'html_class'                      => 'eea-vanco-requestid',
                    'default'                         => date('YmdHis') . '' . rand(0, 9999),
                    'sensitive_data_removal_strategy' => new EE_All_Sensitive_Data_Removal()
                    )),
                'vanco_customerid'       => new EE_Hidden_Input(array(
                    'html_class'                      => 'eea-vanco-customerid',
                    'default'                         => false,
                    'sensitive_data_removal_strategy' => new EE_All_Sensitive_Data_Removal()
                    )),
                'vanco_customerref'      => new EE_Hidden_Input(array(
                    'html_class'                      => 'eea-vanco-customerref',
                    'default'                         => false,
                    'sensitive_data_removal_strategy' => new EE_All_Sensitive_Data_Removal()
                    )),
                'vanco_paymentmethodref' => new EE_Hidden_Input(array(
                    'html_class'                      => 'eea-vanco-paymentmethodref',
                    'default'                         => false,
                    'sensitive_data_removal_strategy' => new EE_All_Sensitive_Data_Removal()
                    )),
                'vanco_select_flag'      => new EE_Hidden_Input(array(
                    'html_id'                         => 'eea_vanco_submit_payment',
                    'default'                         => 'vanco_flag',
                    'sensitive_data_removal_strategy' => new EE_All_Sensitive_Data_Removal()
                    ))
            ),
            'exclude'     => array('phone')
        );

        parent::__construct($payment_method, $options_array);
    }


    /**
     * Sets the defaults for the billing form according to the attendee's details
     * @param int | \EE_Attendee $attendee
     */
    public function populate_from_attendee($attendee)
    {
        parent::populate_from_attendee($attendee);
        $this->populate_defaults(array(
            'state' => $attendee->state_abbrev()
        ));
    }


    /**
     * create_attendee_from_billing_form_data
     * uses info from the billing form to create a new attendee
     * @return \EE_Attendee
     */
    public function create_attendee_from_billing_form_data()
    {
        // grab billing form data
        $data = $this->valid_data();
        return EE_Attendee::new_instance(array(
                'ATT_fname'    => !empty($data['first_name']) ? $data['first_name'] : '',
                'ATT_lname'    => !empty($data['last_name']) ? $data['last_name'] : '',
                'ATT_email'    => !empty($data['email']) ? $data['email'] : '',
                'ATT_address'  => !empty($data['address']) ? $data['address'] : '',
                'ATT_address2' => !empty($data['address2']) ? $data['address2'] : '',
                'ATT_city'     => !empty($data['city']) ? $data['city'] : '',
                'STA_abbrev'   => !empty($data['state']) ? $data['state'] : '',
                'CNT_ISO'      => !empty($data['country']) ? $data['country'] : '',
                'ATT_zip'      => !empty($data['zip']) ? $data['zip'] : '',
                'ATT_phone'    => !empty($data['phone']) ? $data['phone'] : ''
        ));
    }
}

// End of class EEA_Vanco_Billing_Form.
// End of file EEA_Vanco_Billing_Form.form.php