<?php

if (!defined('EVENT_ESPRESSO_VERSION')) {
    exit('No direct script access allowed');
}



/**
 *
 * Class EEG_Vanco_Onsite
 *
 *
 * @package			Event Espresso
 * @subpackage		eea-vanco-payment-method
 * @author			Event Espresso
 *
 */
class EEG_Vanco_Onsite extends EE_Onsite_Gateway
{

    /**
     * 	Vanco API user ID.
     * 	@var string
     */
    protected $_user_id = null;

    /**
     * 	Vanco API Client ID.
     * 	@var string
     */
    protected $_client_id = null;

    /**
     * 	Vanco API user password.
     * 	@var string
     */
    protected $_user_password = null;

    /**
     * 	Unique Vanco Encryption Key.
     * 	@var string
     */
    protected $_encryption_key = null;

    /**
     * 	Accept only Debit Cards?.
     * 	@var string
     */
    protected $_is_debitcard_only = false;

    /**
     * 	Vanco Session ID.
     * 	@var string
     */
    protected $_vanco_session_id = false;

    /**
     * 	Currencies supported by this gateway.
     * 	@var array
     */
    protected $_currencies_supported = array('USD');


    /**
     *
     * 	@param EEI_Payment  $payment
     * 	@param array        $billing_info
     * 	@return \EE_Payment|\EEI_Payment
     */
    public function do_direct_payment($payment, $billing_info = null)
    {
        if (!isset($billing_info['vanco_customerref']) || empty($billing_info['vanco_customerref'])) {
            return $payment;
        }
        if ($payment instanceof EEI_Payment) {

            $request_url = $this->_debug_mode
                ? 'https://uat.vancopayments.com/cgi-bin/wsnvp.vps'
                : 'https://myvanco.vancopayments.com/cgi-bin/wsnvp.vps';
            // Gather all the request needed data.
            //$request_id = date( 'YmdHis' ) . '' . rand(0, 9999);
            $debitcard_only = $this->_is_debitcard_only ? 'Yes' : 'No';
            // If a Start Date is passed as 0000-00-00, the transaction has a frequency of One Time, and there is no End Date sent,
            // Vanco’s system will determine the next possible start date for you.
            $nvp_vars = 'requesttype=eftaddcompletetransaction&isdebitcardonly=' . $debitcard_only
                . '&customerref=' . $billing_info['vanco_customerref']
                . '&paymentmethodref=' . $billing_info['vanco_paymentmethodref']
                . '&amount=' . number_format($payment->amount(), 2)
                . '&startdate=0000-00-00&frequencycode=O';
            $nvp_vars = preg_replace('/\s+/', '', $nvp_vars);
            $nvp_var = EE_PMT_Vanco_Onsite::vanco_encrypt_nvpvar($nvp_vars);
            $request_vars = 'sessionid=' . $this->_vanco_session_id . '&nvpvar=' . $nvp_var;
            // Put the URL together.
            $params = array(
                'method'      => 'POST',
                'timeout'     => 60,
                'redirection' => 5,
                'blocking'    => true,
                'headers'     => array(
                    'Content-Type' => 'application/x-www-form-urlencoded'
                ),
                'body'        => $request_vars,
                'cookies'     => array()
            );
            $this->log(
                array(
                    'nvp_vars before encryption:' => $nvp_vars,
                    'Vanco request URL:'          => $request_url,
                    'Vanco request parameters:'   => $params
                ),
                $payment
            );
            // Vanco POST request.
            $vanco_response = wp_remote_post($request_url, $params);
            // Process Vanco Response.
            if (!is_wp_error($vanco_response) && (strpos($vanco_response['body'], 'nvpvar') !== false)) {
                //Save the response from Vanco.
                $payment->set_details($vanco_response['body']);

                //Decrypt the nvpvars
                $nvpvar = $vanco_response['body'];
                $dec_nvpvar = EE_PMT_Vanco_Onsite::vanco_decrypt_nvpvar(str_replace('nvpvar=', '', $nvpvar));
                $nvparray = array();
                parse_str($dec_nvpvar, $nvparray);

                //Set the TXN ID for the pay as the transactionref from Vanco.
                $payment->set_txn_id_chq_nmbr($nvparray['transactionref']);

                if (array_key_exists('errorlist', $nvparray) && (!empty($nvparray['errorlist']))) {
                    $error_mapping = EEH_File::get_file_contents(EEA_VANCO_PM_PATH . 'error-code-mapping.json');
                    $error_list = json_decode($error_mapping, true);

                    $payment->set_status($this->_pay_model->declined_status());
                    $errorlist = explode(',', $nvparray['errorlist']);
                    $gateway_response = '';
                    foreach ($errorlist as $id) {
                        $gateway_response .= $error_list[$id];
                        if ($id === '434' && array_key_exists('ccauthdesc', $nvparray)) {
                            $gateway_response .= ': ' . $nvparray['ccauthdesc'] . '; ';
                        } else {
                            $gateway_response .= '; ';
                        }
                    }
                    $payment->set_gateway_response($gateway_response);
                    $this->log(array('Vanco Error:' => $nvparray, 'Error text:' => $gateway_response), $payment);
                } else {
                    $payment->set_status($this->_pay_model->approved_status());
                    $this->log(
                        array(
                            'Vanco Payment Accepted:' => array(
                                'Billing info:' => $billing_info,
                                'NVPs:'         => $nvparray
                            )
                        ),
                        $payment
                    );
                }
            } else {
                $this->log(array('Vanco Request Error:' => $vanco_response), $payment);
                $payment->set_status($this->_pay_model->failed_status());
            }
        }
        return $payment;
    }
}

// End of class EEG_Vanco_Onsite.
// End of file EEG_Vanco_Onsite.pm.php