<?php

if (!defined('EVENT_ESPRESSO_VERSION')) {
	exit('No direct script access allowed');
}

/**
 *
 * EEG_TransFirst_Onsite
 *
 * Just approves payments where billing_info[ 'credit_card' ] == 1.
 * If $billing_info[ 'credit_card' ] == '2' then its pending.
 * All others get refused
 *
 * @package			Event Espresso
 * @subpackage
 * @author				Mike Nelson
 *
 */
class EEG_TransFirst_Onsite extends EE_Onsite_Gateway{

	/**
	 * All the currencies supported by this gateway. Add any others you like,
	 * as contained in the esp_currency table
	 * @var array
	 */
	protected $_currencies_supported = array(
					'USD'
	);
	
	protected $_gateway_id;
	
	protected $_regkey;
	
	const response_code_approved = 0;
	const response_code_partially_approved = 10;


	/**
	 *
	 * @param EEI_Payment $payment
	 * @param array $billing_info
	 * @return \EE_Payment|\EEI_Payment
	 */
	public function do_direct_payment($payment, $billing_info = null) {
		$this->_log_clean_request( $billing_info, $payment );
		
		require_once dirname(__FILE__) . '/lib/TransFirst/TransFirstAutoload.php';
		$transFirstServiceSend = new TransFirstServiceSend(
			array(
				TransFirstServiceSend::WSDL_URL => $this->_debug_mode ? 
					'https://ws.cert.transactionexpress.com/portal/merchantframework/MerchantWebServices-v1?wsdl' : 
					'https://ws.transactionexpress.com/portal/merchantframework/MerchantWebServices-v1?wsdl',
				TransFirstServiceSend::WSDL_EXCEPTIONS => 1,
				TransFirstServiceSend::WSDL_CACHE_WSDL => false
			)
		);
		// sample call for TransFirstServiceSend::SendTran()
		$merchant = new TransFirstStructMerc( 
			$this->_gateway_id, //gateway id
			$this->_regkey //reg key
		);
		$merchant->setInType( 1 );
		$merchant->setProdType( 5 );

		$contact = new TransFirstStructContact();
		$contact->setFullName( $billing_info[ 'full_name' ] );
		if( $billing_info[ 'phone' ] ) {
			$contact->setPhone(
				new TransFirstStructPhone( 
					2,//phone type, "mobile"
					$billing_info[ 'phone' ]
				)	
			);
		}
		$contact->setZipCode( $billing_info[ 'zip' ] );
		$contact->setCtry( 'US' );
		$contact->setEmail( $billing_info[ 'email'] );
		$contact->setAddrLn1( $billing_info[ 'address' ] );
		if( $billing_info[ 'address2' ] ) {
			$contact->setAddrLn2( $billing_info[ 'address2' ] );
		}
		$contact->setCity( $billing_info[ 'city'] );
		//note: we've setup the form and payment method type class
		//so the billing info state sent to this gateway class is actually
		//the state abbreviation, so it's ready to use here
		$contact->setState( $billing_info[ 'state' ] );
		$req = new TransFirstStructSendTranRequest( 
			$merchant
		);
		$req->setContact( $contact );
		$req->setReqAmt( $payment->amount() * 100 );
		$req->setIndCode( 2 );
		$authreq = new TransFirstStructAuthReq();
		$authreq->setOrdNr( 
			apply_filters( 
				'FHEE__EEG_TransFirst_Onsite__do_direct_payment__authReqOrdNr', 'event-espresso-payment-' . $payment->ID(), 
				$payment, 
				$billing_info 
			) 
		);
		$req->setAuthReq( $authreq );
		if( $billing_info[ 'payment_type' ] === 'credit' ) {
			$req->setTranCode( 1 );//credit card auth & settle
			$card = new TransFirstStructCard();
			$credit_card_info = $billing_info[ 'credit' ];
			$card->setPan( $credit_card_info[ 'credit_card'] );
			$card->setSec( $credit_card_info[ 'cvv' ] );
			$card->setXprDt( $credit_card_info[ 'exp_year' ] . $credit_card_info[ 'exp_month'] );
			$card->setDbtOrCdt( 1 );
			$req->setCard( $card );
		} else {
			$req->setTranCode( 11 );//ach debit
			$echeck_info = $billing_info[ 'echeck' ];
			$achinfo = new TransFirstStructAchEcheck();
			$achinfo->setBankRtNr( $echeck_info[ 'routing_number' ] );		
			$achinfo->setAcctNr( $echeck_info[ 'account_number' ] );
			$achinfo->setAcctType( $echeck_info[ 'account_type' ] );
			$achinfo->setSeccCode( 1 );//WEB payment
			$req->setAchEcheck( $achinfo );
		}
		
		
		$result = $transFirstServiceSend->SendTran( $req );
		$this->log( array( 'stage' => 'response received', 'response_data' => print_r( $result, true ) ), $payment );

		if( $result ){
			$result = $transFirstServiceSend->getResult();
			if( $result instanceof TransFirstStructSendTranResponse ) {
				if( in_array( 
						$result->getRspCode(),
						array(
							EEG_TransFirst_Onsite::response_code_approved,
							EEG_TransFirst_Onsite::response_code_partially_approved
						)
					)
				) {
					$payment->set_status( $this->_pay_model->approved_status() );
					//update the payment amount, realizing they send the amount in pennies
					$payment->set_amount( $result->getTranData()->getAmt() / 100 );
				} else {
					//somehow it failed
					$payment->set_status( $this->_pay_model->declined_status() );
				}
				$payment->set_gateway_response( $this->_get_response_message_from_code( $result->getRspCode() ) );
				$payment->set_txn_id_chq_nmbr( $result->getTranData()->getTranNr() );
				$payment->set_details( print_r( $result, true ) );
			}
		} else {
			$this->log( array(
				'request_headers' => $transFirstServiceSend->getSoapClient()->__getLastRequestHeaders(),
				'response_headers' => $transFirstServiceSend->getSoapClient()->__getLastResponseHeaders(),
				'response_body' => $transFirstServiceSend->getSoapClient()->__getLastResponse()
			), $payment);
			$payment->set_status( $this->_pay_model->failed_status() );
			$payment->set_gateway_response( __( 'Error occured communicating with TransFirst gateway server', 'event_espresso' ) );
			$payment->set_details( print_r( $transFirstServiceSend->getSoapClient()->__getLastResponse(), true ) );
		}
		return $payment;
	}
	
	/**
	 * Logs the request data
	 * @param array $billing_info
	 * @param EE_Payment $payment
	 */
	protected function _log_clean_request( $billing_info, $payment ) {
		unset( $billing_info[ 'credit' ], $billing_info[ 'echeck' ] );
		$this->log( array( 'stage' => 'initiating request', 'billing_data' => $billing_info  ), $payment );
	}
	
	/**
	 * Gets the response string based off the code, according to transfirst's docuemtnation.
	 * See Transaction Response Codes in "TransactionExpress Integration Guide"
	 * @return string
	 */
	protected function _get_response_message_from_code( $response_code ) {
		$map = array(
			'00' => __( 'Approved or completed successfully', 'event_espresso' ),
			'01' => __( 'Refer to card issuer', 'event_espresso' ),
			'02' => __( 'Refer to card issuer, special condition', 'event_espresso' ),
			'03' => __( 'Invalid merchant', 'event_espresso' ),
			'04' => __( 'Pick-up card', 'event_espresso' ),
			'05' => __( 'Do not honor', 'event_espresso' ),
			'06' => __( 'Error', 'event_espresso' ),
			'07' => __( 'Pick-up card, special condition', 'event_espresso' ),
			'08' => __( 'Declined: Honor with identification(this is a decline response when a card not present transaction)', 'event_espresso' ),
			'09' => __( 'Request in progress', 'event_espresso' ),
			'10' => __( 'Approved, partialauthorization', 'event_espresso' ),
			'11' => __( 'Declined: VIP (this is a decline response fora card not present transaction)', 'event_espresso' ),
			'12' => __( 'Invalid transaction', 'event_espresso' ),
			'13' => __( 'Invalid amount', 'event_espresso' ),
			'14' => __( 'Invalid card number', 'event_espresso' ),
			'15' => __( 'No such issuer', 'event_espresso' ),
			'16' => __( 'Declined: update track 3', 'event_espresso' ),
			'17' => __( 'Customer cancellation', 'event_espresso' ),
			'18' => __( 'Customer dispute', 'event_espresso' ),
			'19' => __( 'Re-enter transaction', 'event_espresso' ),
			'20' => __( 'Invalid response', 'event_espresso' ),
			'21' => __( 'No action taken', 'event_espresso' ),
			'22' => __( 'Suspected malfunction', 'event_espresso' ),
			'23' => __( 'Unacceptable transaction fee', 'event_espresso' ),
			'24' => __( 'File update not supported', 'event_espresso' ),
			'25' => __( 'Unable to locate record', 'event_espresso' ),
			'26' => __( 'Duplicate record', 'event_espresso' ),
			'27' => __( 'File update field edit error', 'event_espresso' ),
			'28' => __( 'File update file locked', 'event_espresso' ),
			'29' => __( 'File update failed', 'event_espresso' ),
			'30' => __( 'Format error', 'event_espresso' ),
			'31' => __( 'Bank not supported', 'event_espresso' ),
			'33' => __( 'Expired card, pick-up', 'event_espresso' ),
			'34' => __( 'Suspected fraud, pick-up', 'event_espresso' ),
			'35' => __( 'Contact acquirer, pick-up', 'event_espresso' ),
			'36' => __( 'Restricted card, pick-up', 'event_espresso' ),
			'37' => __( 'Call acquirer security, pick-up', 'event_espresso' ),
			'38' => __( 'PIN tries exceeded, pick-up', 'event_espresso' ),
			'39' => __( 'No credit account', 'event_espresso' ),
			'40' => __( 'Function not supported', 'event_espresso' ),
			'41' => __( 'Lost card, pick-up', 'event_espresso' ),
			'42' => __( 'No universal account', 'event_espresso' ),
			'43' => __( 'Stolen card, pick-up', 'event_espresso' ),
			'44' => __( 'No investment account', 'event_espresso' ),
			'45' => __( 'Account closed', 'event_espresso' ),
			'46' => __( 'Identification required', 'event_espresso' ),
			'47' => __( 'Identification cross-check required', 'event_espresso' ),
			'48' => __( 'No customer record', 'event_espresso' ),
			'49' => __( 'Reserved for future Realtime use', 'event_espresso' ),
			'50' => __( 'Reserved for future Realtime use', 'event_espresso' ),
			'51' => __( 'Not sufficient funds', 'event_espresso' ),
			'52' => __( 'No checking account', 'event_espresso' ),
			'53' => __( 'No savings account', 'event_espresso' ),
			'54' => __( 'Expired card', 'event_espresso' ),
			'55' => __( 'Incorrect PIN', 'event_espresso' ),
			'56' => __( 'No card record', 'event_espresso' ),
			'57' => __( 'Transaction not permitted to cardholder', 'event_espresso' ),
			'58' => __( 'Transaction notpermitted on terminal', 'event_espresso' ),
			'59' => __( 'Suspected fraud', 'event_espresso' ),
			'60' => __( 'Contact acquirer', 'event_espresso' ),
			'61' => __( 'Exceeds withdrawal limit', 'event_espresso' ),
			'62' => __( 'Restricted card', 'event_espresso' ),
			'63' => __( 'Security violation', 'event_espresso' ),
			'64' => __( 'Original amount incorrect', 'event_espresso' ),
			'65' => __( 'Exceeds withdrawal frequency', 'event_espresso' ),
			'66' => __( 'Call acquirer security', 'event_espresso' ),
			'67' => __( 'Hard capture', 'event_espresso' ),
			'68' => __( 'Response received too late', 'event_espresso' ),
			'69' => __( 'Advice received too late(the response from a request was received too late )', 'event_espresso' ),
			'70' => __( 'Reserved for future use', 'event_espresso' ),
			'71' => __( 'Reserved for future Realtime use', 'event_espresso' ),
			'72' => __( 'Reserved for future Realtime use', 'event_espresso' ),
			'73' => __( 'Reserved for future Realtime use', 'event_espresso' ),
			'74' => __( 'Reserved for future Realtime use', 'event_espresso' ),
			'75' => __( 'PIN tries exceeded', 'event_espresso' ),
			'76' => __( 'Reversal: Unable to locate previous message (no match on Retrieval Reference Number)/Reserved for future Realtime use', 'event_espresso' ),
			'77' => __( 'Previous message located for a repeat or reversal, but repeat or reversal data is inconsistent with original message/Intervene, bank approval required', 'event_espresso' ),
			'78' => __( 'Invalid/non-existent account –Decline (MasterCard specific)/Intervene, bank approval required for partial amount', 'event_espresso' ),
			'79' => __( 'Already reversed (by Switch)/ Reserved for client-specific use (declined)', 'event_espresso' ),
			'80' => __( 'No financial Impact (Reserved for declined debit)/ Reserved for client-specific use (declined)', 'event_espresso' ),
			'81' => __( 'PIN cryptographic error found by the Visa security module during PIN decryption/ Reserved for client-specific use (declined)', 'event_espresso' ),
			'82' => __( 'Incorrect CVV/ Reserved for client-specific use (declined)', 'event_espresso' ),
			'83' => __( 'Unable to verify PIN/ Reserved for client-specific use (declined)', 'event_espresso' ),
			'84' => __( 'Invalid Authorization Life Cycle –Decline (MasterCard) or Duplicate Transaction Detected (Visa)/ Reserved for client-specific use (declined)', 'event_espresso' ),
			'85' => __( 'No reason to decline a request for Account Number Verification or Address Verification/ Reserved for client-specific use (declined)', 'event_espresso' ),
			'86' => __( 'Cannot verify PIN/ Reserved for client-specific use (declined)', 'event_espresso' ),
			'87' => __( 'Reserved for client-specific use (declined)', 'event_espresso' ),
			'88' => __( 'Reserved for client-specific use (declined)', 'event_espresso' ),
			'89' => __( 'Reserved for client-specific use (declined)', 'event_espresso' ),
			'90' => __( 'Cut-off in progress', 'event_espresso' ),
			'91' => __( 'Issuer or switch inoperative', 'event_espresso' ),
			'92' => __( 'Routing error', 'event_espresso' ),
			'93' => __( 'Violation of law', 'event_espresso' ),
			'94' => __( 'Duplicate Transmission (Integrated Debit and MasterCard)', 'event_espresso' ),
			'95' => __( 'Reconcile error', 'event_espresso' ),
			'96' => __( 'System malfunction', 'event_espresso' ),
			'97' => __( 'Reserved for future Realtime use', 'event_espresso' ),
			'98' => __( 'Exceeds cash limit', 'event_espresso' )
		);
		if( isset( $map[ $response_code ] ) ) {
			$response_message_from_transfirst = sprintf( 
				_x( '%1$s (%2$s)', 'Payment response message (response code)', 'event_espresso' ),
				$map[ $response_code ],
				$response_code
			);
			if( in_array( $response_code, array( '00', '10' ) ) ) {
				return $response_message_from_transfirst;
			} else {
				return sprintf( 
					__( 'Payment declined. %1$s', 'event_espresso' ),
					$response_message_from_transfirst
				);
			}
		} else {
			return sprintf( 
				__( 'Payment failed. An unexpected response code (%1$s) was received.', 'event_espresso' ), 
				$response_code 
			);
		}
	}
}

// End of file EEG_TransFirst_Onsite.php', 'event_espresso' ),
