<?php

if (!defined('EVENT_ESPRESSO_VERSION')) {
	exit('No direct script access allowed');
}

/**
 *
 * EE_PMT_Onsite
 *
 *
 * @package			Event Espresso
 * @subpackage
 * @author				Mike Nelson
 *
 */
class EE_PMT_TransFirst_Onsite extends EE_PMT_Base{

	/**
	 *
	 * @param EE_Payment_Method $pm_instance
	 * @return EE_PMT_TransFirst_Onsite
	 */
	public function __construct($pm_instance = NULL) {
		require_once($this->file_folder().'EEG_TransFirst_Onsite.gateway.php');
		$this->_gateway = new EEG_TransFirst_Onsite();
		$this->_pretty_name = __("TransFirst Transaction Express Onsite", 'event_espresso');
		$this->_requires_https = true;
		parent::__construct($pm_instance);
	}

	/**
	 * Adds the help tab
	 * @see EE_PMT_Base::help_tabs_config()
	 * @return array
	 */
	public function help_tabs_config(){
		return array(
			$this->get_help_tab_name() => array(
				'title' => __('TransFirst Transaction Express Onsite Settings', 'event_espresso'),
				'filename' => 'transfirst_onsite'
				),
		);
	}

	/**
	 * @param \EE_Transaction $transaction
	 * @return \EE_Billing_Attendee_Info_Form
	 */
	public function generate_new_billing_form( EE_Transaction $transaction = null ) {	
		$form = new EE_TransFirst_Form( $this->_pm_instance );
		if( $this->_pm_instance->debug_mode() ) {
			$form = $this->apply_billing_form_debug_settings( $form );
		}
		return $form;
	}

	/**
	 * Gets the form for all the settings related to this payment method type
	 * @return EE_Payment_Method_Form
	 */
	public function generate_new_settings_form() {
		EE_Registry::instance()->load_helper('Template');
		$form = new EE_Payment_Method_Form(
			array(
				'extra_meta_inputs' => array(
					'gateway_id' => new EE_Text_Input(
						array(
							'html_label_text'=>  sprintf(__("Merchant Gateway ID %s", "event_espresso"), $this->get_help_tab_link() ),
							'required' => true,
						)
					),
					'regkey' => new EE_Text_Input(
						array(
							'html_label_text' => sprintf( __( 'Merchant RegKey %s', 'event_espresso' ), $this->get_help_tab_link() ),
							'required' => true
						)
					),
					'accepted_payment_types' => new EE_Checkbox_Multi_Input(
						array(
							'credit' => __( 'Credit Card', 'event_espresso' ),
							'echeck' => __( 'eCheck / ACH', 'event_espresso' )
						),
						array(
							'html_label_text' => sprintf( __( 'Payment Types Accepted on your Merchant Account %s', 'event_espresso' ), $this->get_help_tab_link() ),
							'required' => true,
							'default' => array( 'credit' )
						)
					)
				)
			)
		);
		return $form;
	}
	
	public function apply_billing_form_debug_settings( \EE_Billing_Info_Form $billing_form ) {
		$credit_section = $billing_form->get_subsection( 'credit' );
		if( $credit_section instanceof EE_Form_Section_Proper ) {
			$credit_section->populate_defaults(
				array(
					'credit_card' => '4485896261017708',
					'exp_year' => 20,
					'cvv' => '123'
				)
			);
		}
		$echeck_section = $billing_form->get_subsection( 'echeck' );
		if( $echeck_section instanceof EE_Form_Section_Proper ) {
			$echeck_section->populate_defaults(
				array(
					'routing_number' => '123456780',
					'account_number' => '12345678901',
					'account_type' => 0
				));
		}
		
		$billing_form->add_subsections(
			array(
				'debug_blurb' => new EE_Form_Section_HTML_From_Template(
					$this->file_folder() . 'templates' . DS . 'transfirst_onsite_debug_info.template.php'
				)
			)
		);
		return $billing_form;
	}
	
	/**
	 * Override the parent so we get the states abbreviation
	 * @param EE_Billing_Info_Form $billing_form
	 * @return array
	 */
	protected function _get_billing_values_from_form( $billing_form ){
		$values = $billing_form->input_values( true, false );
		$state_input = $billing_form->get_input( 'state' );
		if( $state_input instanceof EE_State_Select_Input )  {
			$state_abbrev = EEM_State::instance()->get_var( array( array( 'STA_ID' => $state_input->normalized_value() ) ), 'STA_abbrev' );
			$values[ 'state' ] = $state_abbrev;
		}
		return $values;
	}
}
// End of file EE_PMT_Onsite.php