<?php
if ( ! defined('EVENT_ESPRESSO_VERSION')) {
    exit('No direct script access allowed');
}



/**
 * Class EE_TransFirst_Form
 * Description
 *
 * @package       Event Espresso
 * @author        Michael Nelson
 * @since         $VID:$
 */
class EE_TransFirst_Form extends EE_Billing_Attendee_Info_Form {
    public function __construct( \EE_Payment_Method $payment_method, array $options_array = array() )
    {
        if( ! isset( $options_array['subsections'] ) ) {
            $options_array['subsections'] = array();
        }
        $options_array['subsections'] = array_merge(
            $options_array['subsections']
            ,array(
                'credit' => new EE_Form_Section_Proper(
                    array(
                        'subsections' => array(
                            'credit_card' => new EE_Credit_Card_Input(
                                array(
                                    'required' => false,//this input will be conditionally required, not always required
                                    'html_label_text' => __( 'Credit Card Number', 'event_espresso' ),
                                )
                            ),
                            'exp_month'   => new EE_Credit_Card_Month_Input(
                                true, //yes to leading zero
                                array(
                                    'html_label_text' => __( 'Expiry Month', 'event_espresso' ),
                                ),
                                true //january is 1, as usual
                            ),
                            'exp_year'    => new EE_Credit_Card_Year_Input(
                                array(
                                    'html_label_text' => __( 'Expiry Year', 'event_espresso' ),
                                ),
                                false//2 digit year input
                            ),
                            'cvv'         => new EE_CVV_Input(
                                array(
                                    'html_label_text' => __( 'CVV', 'event_espresso' ),
                                )
                            )
                        ),
                        'layout_strategy' => new EE_Div_Per_Section_Layout()
                    )
                ),
                'echeck' => new EE_Form_Section_Proper(
                    array(
                        'subsections' => array(
                            'routing_number' => new EE_Text_Input(
                                array(
                                    'html_label_text' => __( 'Bank Routing Number', 'event_espresso' ),
                                    'validation_strategies' => array(
                                        new EE_Max_Length_Validation_Strategy(
                                            __( 'Bank Routing Numbers should have no more than 9 digits', 'event_espresso'), 9
                                        ),
                                    ),
                                    //'sensitive_data_removal_strategy' => new EE_Credit_Card_Sensitive_Data_Removal()
                                )
                            ),
                            'account_number' => new EE_Text_Input(
                                array(
                                    'html_label_text' => __( 'Account Number', 'event_espresso' ),
                                    'validation_strategies' => array(
                                        new EE_Max_Length_Validation_Strategy( __( 'Customer Checking/Savings Account numbers should have no more than 17 digits'), 17 ),
                                        new EE_Min_Length_Validation_Strategy( __( 'Customer Checking/Savings Account numbers should have no fewer than 4 digits'), 4 ),
                                    ),
                                    'sensitive_data_removal_strategy' => new EE_Credit_Card_Sensitive_Data_Removal()
                                )
                            ),
                            'account_type'    => new EE_Select_Input(
                                array(
                                    '' => '',
                                    '0' => __( 'Checking', 'event_espresso' ),
                                    '1' => __('Savings', 'event_espresso' ),
                                ),
                                array(
                                    'html_label_text' => __( 'Account Type', 'event_espresso' ),
                                )
                            ),
                        ),
                        'layout_strategy' => new EE_Div_Per_Section_Layout()
                    )
                ),
                'payment_type' => new EE_Hidden_Input(),
            )
        );
        add_filter( 'FHEE__EE_Billing_Attendee_Info_Form__populate_from_attendee', array( 'EE_TransFirst_Form', 'modify_autofilling_attendee_billing_form' ), 10, 3 );

        parent::__construct($payment_method, $options_array);

        //if this isn't the transaction admin page, then use the settings page to determine what subsections to show
        if( ! $this->_displaying_billing_info_submission() ) {
            $settings_form = $payment_method->type_obj()->settings_form();
            $accepted_payment_types = $settings_form->get_input_value('accepted_payment_types');
            $this->_remove_card_or_echeck_section($accepted_payment_types);
        }

        $this->add_subsections(
            array(
                'full_name' => new EE_Text_Input(
                    array(
                        'required' => true,
                        'html_label_text' => __( 'Full Name', 'event_espresso' ),
                        'validation_strategies' => array(
                            new EE_Max_Length_Validation_Strategy( null, 61 )
                        )
                    )
                )
            )
        );
        $this->_set_max_input_length_on_default_inputs();
        $state_input = $this->get_input( 'state' );
        if( $state_input instanceof EE_State_Select_Input ) {
            $this->_customize_state_input( $state_input );
        }
        $this->_exclude_unnecessary_inputs( $this );
        //for all inputs besides phone and email, make sure we only include characters allowed by transfirst
        $allowed_special_chars = array(
            ' ',
            '!',
            '#',
            '$',
            '%',
            '(',
            ')',
            '*',
            '+',
            ',',
            '-',
            '.',
            '/',
            ':',
            ';',
            '?',
            '[',
            ']',
            '^',
            '_',
            '`',
            '{',
            '}',
            '~'
        );
        foreach( $this->inputs_in_subsections() as $form_input ) {
            if( ! $form_input instanceof EE_Email_Input ) {
                $allowed_inputs_for_email = $allowed_special_chars;
            } else {
                $allowed_inputs_for_email = array_merge( $allowed_special_chars, array( '@' ) );
            }
            $form_input->add_validation_strategy(
                new EE_Text_Validation_Strategy(
                    sprintf(
                        __( 'These are the only special characters allowed: %1$s', 'event_espresso' ),
                        implode(', ', $allowed_inputs_for_email )
                    ),
                    '"^([\w\d' . preg_quote( implode( '', $allowed_inputs_for_email ), '"' ) . ']*)$"')
            );
        }
        $this->add_subsections(
            array(
                'phone' => new EE_Phone_Input(
                    array(
                        'html_label_text' => __( 'US Phone', 'event_espresso' )
                    )
                ),
            ),
            'zip',
            false
        );
    }

    /**
     * Changes default billing attendee info inputs' default max length
     */
    protected function _set_max_input_length_on_default_inputs() {
        $extra_max_input_validations_to_add = array(
            'email' => 75,
            'address' => 50,
            'address2' => 50,
            'city' => 40,
            'zip' => 9,
        );
        foreach( $extra_max_input_validations_to_add as $input_name => $max_length_to_add ) {
            $this->get_input( $input_name )->add_validation_strategy( new EE_Max_Length_Validation_Strategy( null, $max_length_to_add ) );
        }
    }

    /**
     * Customizes the state input so that its values are acceptable values for transfirst.
     * Modifies the original input
     * @param EE_State_Select_Input $state_input
     * @return void modifies original form
     */
    protected function _customize_state_input( EE_State_Select_Input $state_input ) {
        $us_state_options = array(
            __( 'United States', 'event_espresso' ) => array(
                '' => ''
            )
        );
        $us_states = EEM_State::instance()->get_all(
            array(
                array(
                    'CNT_ISO' => 'US'
                ),
                'order_by' => array(
                    'STA_name' => 'ASC'
                ),
                'force_join' => array(
                    'Country'
                )
            )
        );
        //use state ABBREVIATIONS as keys instead of the normal IDs
        foreach( $us_states as $us_state ) {
            $us_state_options[ $us_state->country()->name() ][ $us_state->ID() ] = $us_state->name();
        }
        $state_input->set_select_options( $us_state_options );

    }

    /**
     * Excludes default inputs transfirst doesn't need, like first name, last name,
     * country, and phone (because we replace phone with a US phone input)
     */
    protected function _exclude_unnecessary_inputs() {
        $this->exclude(
            array(
                //we just want a full name input
                'first_name',
                'last_name',
                //we don't need country because transfirst only accepts US states
                'country',
                //we want to override the phone input
                'phone'
            )
        );
    }



    /**
     * Also auto fill the full name field
     * @param $default_values
     * @param $attendee
     * @param $billing_form
     * @return array
     */
    public static function modify_autofilling_attendee_billing_form( $default_values, $attendee, $billing_form ) {
        if( $billing_form instanceof EE_Billing_Attendee_Info_Form
            && $billing_form->name() == 'TransFirst_Onsite_Form'
            && $attendee instanceof EE_Attendee
            && is_array( $default_values )
        ) {
            $default_values[ 'full_name' ] = $attendee->full_name();
        }
        return $default_values;
    }



    /**
     * Removes the card or echeck section, depending on what are the accepted payment methods inputted
     * @param array $accepted_payment_types
     */
    protected function _remove_card_or_echeck_section( $accepted_payment_types = array()) {
        $credit = $this->get_subsection( 'credit' );
        $echeck = $this->get_subsection( 'echeck' );
        if( count( $accepted_payment_types ) === 1 ) {
            //only one type of payment accepted
            $accepted_payment_type = reset( $accepted_payment_types );
            if( $accepted_payment_type === 'credit' ) {
                $this->exclude( array( 'echeck' ) );
                foreach( $credit->inputs() as $input_obj ) {
                    $input_obj->set_required();
                }
            } else {
                $this->exclude(array('credit'));
                foreach ($echeck->inputs() as $input_obj) {
                    $input_obj->set_required();
                }
            }
            $this->get_input( 'payment_type' )->set_default( $accepted_payment_type );
        } else {
            //both credit card and echeck accepted
            foreach( $credit->inputs() as $input_obj ) {
                $input_obj->add_validation_strategy(
                    new EE_Conditionally_Required_Validation_Strategy(
                        null,
                        array( '../../payment_type' => array( '=', 'credit' ) )
                    )
                );
            }
            foreach( $echeck->inputs() as $input_obj ) {
                $input_obj->add_validation_strategy(
                    new EE_Conditionally_Required_Validation_Strategy(
                        null,
                        array( '../../payment_type' => array( '=', 'echeck' ) )
                    )
                );
            }
            $this->exclude( array( 'payment_type' ) );
            $this->add_subsections(
                array(
                    'payment_type' => new EE_Select_Reveal_Input(
                        array(
                            '' => '',
                            'credit' => __( 'Credit Card', 'event_espresso' ),
                            'echeck' => __( 'eCheck', 'event_espresso' )
                        ),
                        array(
                            'required' => true
                        )
                    ),
                ),
                'phone',
                false
            );
        }
    }



    /**
     * Returns whether or not we're displaying the billing info submission data
     * (right now assumes we only do it on a specific admin page; this won't be sufficient
     * if we change that someday.)
     * @return bool
     */
    protected function _displaying_billing_info_submission(){
        return isset( $_GET['page'] )
                  && isset( $_GET['action'] )
                  && 'espresso_transactions' === $_GET['page']
                  && 'view_transaction' === $_GET['action'];
    }

    protected function _normalize($req_data) {
        parent::_normalize($req_data);
        $this->_remove_card_or_echeck_section( array( $this->get_input_value( 'payment_type' ) ) );
    }
}